/*
// File:		MessageComposer.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2006 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include "MessageComposer.hpp"
#include <assert.h>

/*
// Presence flags for the corresponding parts of the message
// Comments after the constants show what parts of the message they relate to.
*/
enum
{
	kTextFlag		= 1,		/*Plain text part or unicode text part*/
	kHtmlFlag		= 1 << 1,	/*Html part*/
	kInlineFlag		= 1 << 2,	/*Inline part*/
	kAttachmentFlag	= 1 << 3	/*Attachment part*/
};

/*
// Constant types of the multipart/alternative message part
*/
enum
{
	kHtmlPart,
	kAlternativePart
};



// Settings

static EmaiUniChar kMimeVersion[]			= {'1', '.', '0', '\0'};
static EmaiUniChar kType[]					= {'t', 'y', 'p', 'e', 0};
static EmaiUniChar kTextHtml[]				= {'t', 'e', 'x', 't', '/', 'h', 't', 'm', 'l', '\0'};
//static EmaiUniChar kTextPlain[]				= {'t', 'e', 'x', 't', '/', 'p', 'l', 'a', 'i', 'n', 0};
static EmaiUniChar kMultipartAlternative[]	= {'m', 'u', 'l', 't', 'i', 'p', 'a', 'r', 't', '/', 'a', 'l', 't', 'e', 'r', 'n', 'a', 't', 'i', 'v', 'e', '\0'};
static EmaiUniChar kMultipartRelated[]		= {'m', 'u', 'l', 't', 'i', 'p', 'a', 'r', 't', '/', 'r', 'e', 'l', 'a', 't', 'e', 'd', '\0'};
static EmaiUniChar kMultipartMixed[]		= {'m', 'u', 'l', 't', 'i', 'p', 'a', 'r', 't', '/', 'm', 'i', 'x', 'e', 'd', '\0'};
static EmaiUniChar kFileName[]				= {'f', 'i', 'l', 'e', 'n', 'a', 'm', 'e', '\0'};
static EmaiUniChar kName[]					= {'n', 'a', 'm', 'e', '\0'};
static EmaiUniChar kAttachment[]			= {'a', 't', 't', 'a', 'c', 'h', 'm', 'e', 'n', 't', '\0'};
static EmaiUniChar kInline[]				= {'i', 'n', 'l', 'i', 'n', 'e', '\0'};

/***************************************************
//
// MessageComposer class implemetation
//
/***************************************************

/*
// Create message composer object
*/

MessageComposer::MessageComposer():
	mTextPart(NULL, false),
	mHtmlPart(NULL, false)
{
}

/*Dispose message composer object*/
MessageComposer::~MessageComposer()
{
}

/*Sets the plain text message part*/
/*Parameters:*/
/*	inPlainText - pointer to the plain text data*/
void
MessageComposer::SetText(
		EmaiConstCharPtr inPlainText,
		EmaiUint32 inCharacterSet
	)
{
	/*Set the properties for the new message part*/
	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = NULL;
	partData.contentType = NULL;
	partData.contentDisposition = NULL;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartPlainTextData;
	partData.partData.plainText.text = inPlainText;
	partData.partData.plainText.textSize = static_cast<EmaiUint32>(strlen(inPlainText));
	partData.partData.plainText.characterSet = inCharacterSet;
	
	/*Create the text part*/
	mTextPart = Emai::MessagePart(partData);
}


/*Set the unicode text message part*/
/*Parameters:*/
/*	inUnicodeText - pointer to the unicode text data*/
void
MessageComposer::SetUnicodeText(
		EmaiConstUniCharPtr inUnicodeText
	)
{
	/*Set the properties for the new message part*/
	EmaiPartData partData = {EmaiStructureVersionFirst};
	
	partData.contentID = NULL;
	partData.contentType = NULL;
	partData.contentDisposition = NULL;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartUnicodeTextData;
	partData.partData.unicodeText.text = inUnicodeText;
	partData.partData.unicodeText.textSize = static_cast<EmaiUint32>(
								std::char_traits<EmaiUniChar>::length(inUnicodeText));
	
	/*Create the text part*/
	mTextPart = Emai::MessagePart(partData);
}


/*Sets the html message part*/
/*Parameters:*/
/*	inHtmlText - pointer to the html text data*/
void
MessageComposer::SetHtml(
		EmaiConstCharPtr inHtmlText
	)
{
	/*Set the properties for the new message part*/
	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = NULL;
	partData.contentType = NULL;
	partData.contentDisposition = NULL;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartHTMLData;
	partData.partData.htmlText.html = inHtmlText;
	partData.partData.htmlText.htmlSize = static_cast<EmaiUint32>(strlen(inHtmlText));

	/*Create the html part*/
	mHtmlPart = Emai::MessagePart(partData);
}	

/*Add the attachment data to the list of the attachments*/
/*Parameters:*/
/*	inData - pointer to the data*/
/*	inDataSize - size of the data*/
/*	inFileName - pointer to the name of the data*/
void
MessageComposer::AddAttachmentFromData(
		const void* inData,
		EmaiUint32 inDataSize,
		EmaiConstUniCharPtr inFileName
	)
{
	/*Check if the input parameters are correct*/
	if (inData == NULL || inDataSize == 0 || inFileName == NULL)
	{
		return;
	}
	
	/*Set the properties for the new message part*/
	Emai::ContentField contentType(NULL);
	Emai::ContentField contentDisposition(kAttachment);
	contentType.AddParameter(kName, inFileName);
	contentDisposition.AddParameter(kFileName, inFileName);
	
	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = NULL;
	partData.contentType = contentType;
	partData.contentDisposition = contentDisposition;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartDataData;
	partData.partData.data.data = inData;
	partData.partData.data.dataSize = inDataSize;

	/*Add the attachment part to the list*/
	mAttachments.push_back(Emai::MessagePart(partData));
}

/*Add the attachment file to the list of the attachments*/
/*Parameters:*/
/*	inFileName - pointer to the name of the file*/
/*	inFileFullPath - pointer to the full path of the file*/
void
MessageComposer::AddAttachmentFromFile(
		EmaiConstUniCharPtr inFileName,
		EmaiConstUniCharPtr inFileFullPath
	)
{
	/*Check if the input parameters are correct*/
	if (inFileName == NULL || inFileFullPath == NULL)
	{
		return;
	}

	/*Set the properties for the new message part*/
	Emai::ContentField contentType(NULL);
	Emai::ContentField contentDisposition(kAttachment);
	contentType.AddParameter(kName, inFileName);
	contentDisposition.AddParameter(kFileName, inFileName);

	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = NULL;
	partData.contentType = contentType;
	partData.contentDisposition = contentDisposition;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartFileData;
	partData.partData.file.filePath = inFileFullPath;

	/*Add the attachment part to the list*/
	mAttachments.push_back(Emai::MessagePart(partData));
}

/*Add the inline attachment data to the list of the inline attachments*/
/*Parameters:*/
/*	inData - pointer to the inline data*/
/*	inDataSize - size of the inline data*/
/*	inFileName - pointer to the name of the inline data*/
/*	inContentId - pointer to the inline content identifier*/
void
MessageComposer::AddInlineFromData(
		const void* inData,
		EmaiUint32 inDataSize,
		EmaiConstUniCharPtr inFileName,
		EmaiConstUniCharPtr inContentId = NULL
	)
{
	/*Check if the input parameters are correct*/
	if (inData == NULL || inDataSize == 0 || inFileName == NULL)
	{
		return;
	}
	
	/*Set the properties for the new message part*/
	Emai::ContentField contentType(NULL);
	Emai::ContentField contentDisposition(kInline);
	contentType.AddParameter(kName, inFileName);
	contentDisposition.AddParameter(kFileName, inFileName);

	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = inContentId;
	partData.contentType = contentType;
	partData.contentDisposition = contentDisposition;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartDataData;
	partData.partData.data.data = inData;
	partData.partData.data.dataSize = inDataSize;

	/*Add the inline message part to the list*/
	mInlines.push_back(Emai::MessagePart(partData));
}

/*Add the inline attachment from file to the list of the inline attachments*/
/*Parameters:*/
/*	inFileName - pointer to the name of the file*/
/*	inFileFullPath - pointer to the file full path*/
/*	inContentId - pointer to the inline content identifier*/
void MessageComposer::AddInlineFromFile(
		EmaiConstUniCharPtr inFileName,
		EmaiConstUniCharPtr inFileFullPath,
		EmaiConstUniCharPtr inContentId = NULL
	)
{
	/*Check if the input parameters are correct*/
	if (inFileName == NULL || inFileFullPath == NULL)
	{
		return;
	}
	
	/*Set the properties for the new message part*/
	Emai::ContentField contentType(NULL);
	Emai::ContentField contentDisposition(kInline);
	contentType.AddParameter(kName, inFileName);
	contentDisposition.AddParameter(kFileName, inFileName);

	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = inContentId;
	partData.contentType = contentType;
	partData.contentDisposition = contentDisposition;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartFileData;
	partData.partData.file.filePath = inFileFullPath;

	/*Add the inline message part to the list*/
	mInlines.push_back(Emai::MessagePart(partData));
}

/*Composes message*/
/*Parameters:*/
/*	inFromContact - pointer to vector of contact fields "From"*/
/*	inSenderContact - pointer to sender contact field*/
/*	inToContact - pointer to vector of contact fields "TO"*/
/*	inCCContact - pointer to vector of contact fields "CC"*/
/*	inBCContact - pointer to vector of contact fields "BC"*/
/*	inMessageSubject - pointer to message subject*/
Emai::Message
MessageComposer::ComposeMessage(
		Emai::ContactList inFromContact,
		Emai::ContactList inSenderContact,
		Emai::ContactList inToContact,
		Emai::ContactList inCCContact,
		Emai::ContactList inBCContact,
		EmaiConstUniCharPtr inMessageSubject
	)
{
	
	/*Determide what kind of parts should be included to the message*/
	EmaiUint32 flag = ((EmaiMessagePart)mTextPart != NULL) ? kTextFlag : 0;
	flag |= ((EmaiMessagePart)mHtmlPart != NULL) ? kHtmlFlag : 0;
	flag |= (mInlines.size() != 0) ? kInlineFlag : 0;
	flag |= (mAttachments.size() != 0) ? kAttachmentFlag : 0;

	/*Declaration of the message*/
	Emai::Message message(NULL, false);

	/*Declaration of the complementary variables*/
	Emai::MessagePart rootMessagePart(NULL, false);
	Emai::MessagePart alternativeMessagePart(NULL, false);
	Emai::MessagePart relatedMessagePart(NULL, false);

	/*Examine all possible variants*/
	switch (flag)
	{
		case kTextFlag:
			message = Emai::Message(mTextPart);
			break;

		case kHtmlFlag:
			message = Emai::Message(mHtmlPart);
			break;

		case kInlineFlag:
			rootMessagePart = CreateMixedMultipart();
			BindAttachments(rootMessagePart, mInlines);
			break;

		case kAttachmentFlag:
			rootMessagePart = CreateMixedMultipart();
			BindAttachments(rootMessagePart, mAttachments);
			break;
			
		case kTextFlag | kHtmlFlag:
		{
			EmaiMessagePart messageParts[2] = {mTextPart, mHtmlPart};
			rootMessagePart = CreateAlternativeMultipart();
			rootMessagePart.AddSubParts(messageParts, 2);
			break;
		}

		case kTextFlag | kInlineFlag:
		{
			EmaiMessagePart messagePart = mTextPart;
			rootMessagePart = CreateMixedMultipart();
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mInlines);
			break;
		}

		case kTextFlag | kAttachmentFlag:
		{
			EmaiMessagePart messagePart = mTextPart;
			rootMessagePart = CreateMixedMultipart();
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mAttachments);
			break;
		}

		case kHtmlFlag | kInlineFlag:
		{
			EmaiMessagePart messagePart = mHtmlPart;
			rootMessagePart = CreateRelatedMultipart(kHtmlPart);
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mInlines);
			break;
		}

		case kHtmlFlag | kAttachmentFlag:
		{
			EmaiMessagePart messagePart = mHtmlPart;
			rootMessagePart = CreateMixedMultipart();
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mAttachments);
			break;
		}

		case kInlineFlag | kAttachmentFlag:
			rootMessagePart = CreateMixedMultipart();
			BindAttachments(rootMessagePart, mInlines);
			BindAttachments(rootMessagePart, mAttachments);
			break;

		case kTextFlag | kHtmlFlag | kInlineFlag:
		{
			EmaiMessagePart messageParts[2] = {mTextPart, mHtmlPart};
			alternativeMessagePart = CreateAlternativeMultipart();
			alternativeMessagePart.AddSubParts(messageParts, 2);
			rootMessagePart = CreateRelatedMultipart(kAlternativePart);

			EmaiMessagePart messagePart = alternativeMessagePart;
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mInlines);
			break;
		}

		case kTextFlag | kHtmlFlag | kAttachmentFlag:
		{
			EmaiMessagePart messageParts[2] = {mTextPart, mHtmlPart};
			alternativeMessagePart = CreateAlternativeMultipart();
			alternativeMessagePart.AddSubParts(messageParts, 2);

			rootMessagePart = CreateMixedMultipart();
			EmaiMessagePart messagePart = alternativeMessagePart;
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mAttachments);
			break;
		}

		case kTextFlag | kInlineFlag | kAttachmentFlag:
		{
			rootMessagePart = CreateMixedMultipart();
			EmaiMessagePart messagePart = mTextPart;
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mInlines);
			BindAttachments(rootMessagePart, mAttachments);
			break;
		}

		case kHtmlFlag | kInlineFlag | kAttachmentFlag:
		{
			relatedMessagePart = CreateRelatedMultipart(kHtmlPart);
			EmaiMessagePart messagePart = mHtmlPart;
			relatedMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(relatedMessagePart, mInlines);

			rootMessagePart = CreateMixedMultipart();
			messagePart = relatedMessagePart;
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mAttachments);
			break;
		}

		case kTextFlag | kHtmlFlag | kInlineFlag | kAttachmentFlag:
		{
			EmaiMessagePart messageParts[2] = {mTextPart, mHtmlPart};
			alternativeMessagePart = CreateAlternativeMultipart();
			alternativeMessagePart.AddSubParts(messageParts, 2);

			relatedMessagePart = CreateRelatedMultipart(kAlternativePart);
			EmaiMessagePart messagePart = alternativeMessagePart;
			relatedMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(relatedMessagePart, mInlines);

			rootMessagePart = CreateMixedMultipart();
			messagePart = relatedMessagePart;
			rootMessagePart.AddSubParts(&messagePart, 1);
			BindAttachments(rootMessagePart, mAttachments);
			break;
		}

		default:
			assert(false);
			break;
	}

	/*Create the out message*/
	if (rootMessagePart != NULL)
		message = Emai::Message(rootMessagePart);

	/*Add the different fields to the message*/
	if (message != NULL)
	{
		/*Add the contact fields to the message*/
		message.SetContactList(inFromContact, EmaiFrom);

		if (inSenderContact != NULL)
			message.SetContactList(inSenderContact, EmaiSender);
		if (inToContact != NULL)
			message.SetContactList(inToContact, EmaiTo);
		if (inCCContact != NULL)
			message.SetContactList(inCCContact, EmaiCC);
		if (inBCContact != NULL)
			message.SetContactList(inBCContact, EmaiBCC);
		
		/*Add the subject field*/
		message.SetField(NULL, inMessageSubject, EmaiSubject);

		/*Override default value of the MIME-version field*/
		message.SetField(NULL, kMimeVersion, EmaiMimeVersion);
	}
	
	return message;
}

/*Create the alternative multipart*/
/*Return value:*/
/*	created message part object*/
Emai::MessagePart
MessageComposer::CreateAlternativeMultipart()
{
	/*Set the properties for the new message part*/
	Emai::ContentField contentType(kMultipartAlternative);
	EmaiPartData alternativeEmaiPartData = {EmaiStructureVersionFirst};
		
	alternativeEmaiPartData.contentID = NULL;
	alternativeEmaiPartData.contentType = contentType;
	alternativeEmaiPartData.contentDisposition = NULL;
	alternativeEmaiPartData.encoding = EmaiEncodingAutomatic;
	alternativeEmaiPartData.partType = EmaiPartCompositeData;
	alternativeEmaiPartData.partData.composite.subParts = NULL;
	alternativeEmaiPartData.partData.composite.subPartCount = 0;

	/*Create the alternative message part*/
	return Emai::MessagePart(alternativeEmaiPartData);
}

/*Create the related multipart*/
/*Parameters:*/
/*	parameterValueFlag - child message part type (constant)*/
/*Return value:*/
/*	created message part object*/
Emai::MessagePart
MessageComposer::CreateRelatedMultipart(
		EmaiUint32 parameterValueFlag
	)
{
	/*Set the properties for the new message part*/
	Emai::ContentField contentType(kMultipartRelated);

	EmaiConstUniCharPtr parameterValuePtr = NULL;

	/*Determine the type of the child message part in message part's tree.*/
	switch (parameterValueFlag)
	{
		case kHtmlPart:
			parameterValuePtr = kTextHtml;
			break;

		case kAlternativePart:
			parameterValuePtr = kMultipartAlternative;
			break;

		default:
			assert(0);
			break;
	}
	contentType.AddParameter(kType, parameterValuePtr);

	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = NULL;
	partData.contentType = contentType;
	partData.contentDisposition = NULL;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartCompositeData;
	partData.partData.composite.subParts = NULL;
	partData.partData.composite.subPartCount = 0;
	
	/*Create the related message part*/
	return Emai::MessagePart(partData);
}


/*Create the mixed multipart*/
/*Return values:*/
/*	pointer to the message part*/
Emai::MessagePart
MessageComposer::CreateMixedMultipart()
{
	/*Set the properties for the new message part*/
	Emai::ContentField contentType(kMultipartMixed);

	EmaiPartData partData = {EmaiStructureVersionFirst};

	partData.contentID = NULL;
	partData.contentType = contentType;
	partData.contentDisposition = NULL;
	partData.encoding = EmaiEncodingAutomatic;
	partData.partType = EmaiPartCompositeData;
	partData.partData.composite.subParts = NULL;
	partData.partData.composite.subPartCount = 0;
	
	/*Create the mixed message part*/
	return Emai::MessagePart(partData);
}

/*Add the attachment message parts to the message part*/
/*	inMessagePart - reference to the message part*/
/*	inAttachments - reference to the list of the attachments*/
void
MessageComposer::BindAttachments(
		Emai::MessagePart& inMessagePart,
		const MessageComposer::MessagePartArray& inAttachments
	)
{
	/*Add the attachment message parts to the message part*/
	MessageComposer::MessagePartArray::const_iterator iterator = inAttachments.begin();

	while (iterator != inAttachments.end())
	{
		EmaiMessagePart emaiMessagePart = *iterator;
		inMessagePart.AddSubParts(&emaiMessagePart, 1);
		++iterator;
	}
}

