/*!
// @file		EmaiIMAP.h
//
// @brief		IMAP functionality
//
// @version		mail.it library 1.0
//
// @author		(C) 2008-2010 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIIMAP_H__
#define __EMAIIMAP_H__

#include <EmaiTypes.h>
#include <EmaiErrors.h>
#include <EmaiMessage.h>
#include <EmaiProxy.h>

#ifdef __cplusplus
	extern "C" {
#endif

typedef struct EmaiOpaqueImapSession*			EmaiImapSession;


/*
RFC 5301 page 90: sequence-ser, seq-range, seq-number
*/
typedef struct EmaiOpaqueImapSequenceSet*		EmaiImapSequenceSet;

/***************************************************
//
// This section declares the message sequence management
// functionality, which is used with the following IMAP command:
// COPY, FETCH, STORE, UID COPY, UID FETCH, and UID STORE.
//
***************************************************/

/*!
// @brief	Creates empty message sequence set
//
// @param	outSequenceSet - on exit contains sequence set object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetCreate(
		EmaiImapSequenceSet* outSequenceSet,
		EmaiOptions inOptions);

/*!
// @brief	Adds a number to the specified sequence set
//
// @param	inSequenceSet - sequence set object to be modified
// @param	inNumber - a number to be added to sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetAddNumber(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32 inNumber,
		EmaiOptions inOptions);

/*!
// @brief	Remove a number from the specified sequence set
//
// @param	inSequenceSet - sequence set object to be modified
// @param	inNumber - a number to be removed from sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetRemoveNumber(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32 inNumber,
		EmaiOptions inOptions);

/*!
// @brief	Special number value for EmaiImapSequenceSetAddRange functions. The
//			value can used to represent the largest number in use. For instance,
//			EmaiImapSequenceSetAddRange(sequenceSet, 2, EmaiImapMaxSequenceSetNumber,
//			EmaiNullOptions) will add 2:* range to the sequenceSet.
//			Please note that enumeration for such sequence sets will not work.
//
// @see		EmaiImapSequenceSetAddRange
*/
enum
{
	EmaiImapMaxSequenceSetNumber		= 0
};

/*!
// @brief	Adds a range of numbers to the specified sequence set
//
// @param	inSequenceSet - sequence set object to be modified
// @param	inMinNumber - a lower bound of range to be added to sequence set
// @param	inMaxNumber - a upper bound of range to be added to sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetAddRange(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32 inMinNumber,
		EmaiUint32 inMaxNumber,
		EmaiOptions inOptions);

/*!
// @brief	Remove a range of numbers from the specified sequence set
//
// @param	inSequenceSet - sequence set object to be modified
// @param	inMinNumber - a lower bound of range to be removed from sequence set
// @param	inMaxNumber - a upper bound of range to be removed from sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetRemoveRange(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32 inMinNumber,
		EmaiUint32 inMaxNumber,
		EmaiOptions inOptions);

/*!
// @brief	Adds all numbers from a sequence set object to another sequence set
//
// @param	inSequenceSet - sequence set object to be modified
// @param	inSequenceSetToAdd - a sequence set to be added to sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetAddSequenceSet(
		EmaiImapSequenceSet inSequenceSet,
		EmaiImapSequenceSet inSequenceSetToAdd,
		EmaiOptions inOptions);

/*!
// @brief	Remove all numbers contained in a sequence set object from another
//			sequence set
//
// @param	inSequenceSet - sequence set object to be modified
// @param	inSequenceSetToRemove - a sequence set to be removed from sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetRemoveSequenceSet(
		EmaiImapSequenceSet inSequenceSet,
		EmaiImapSequenceSet inSequenceSetToRemove,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves count of numbers in sequence set.
//
// @param	inSequenceSet - sequence set object to be examined
// @param	outCount - on exit contains count of numbers in the sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetGetCount(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32* outCount,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves first and/or last numbers in sequence set.
//			EmaiErrIncorrectParameter error code will be returned by the
//			function if outFirstNumber and outLastNumber are both NULL.
//			EmaiErrIncorrectParameter is also returned is sequence set is empty
//
// @param	inSequenceSet - sequence set object to be examined
// @param	outFirstNumber - if non-NULL on exit contains first number of the
						sequence set
// @param	outLastNumber - if non-NULL on exit contains last number of the
						sequence set
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetGetBounds(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32* outFirstNumber,		/*can be NULL*/
		EmaiUint32* outLastNumber,		/*can be NULL*/
		EmaiOptions inOptions);


/*!
// @brief	Options for EmaiImapSequenceGetNextNumber function
//
// @see		EmaiImapSequenceSetGetNextNumber
*/
enum EmaiEnumImapSequenceSetGetNextNumberOptions
{
	EmaiImapForward				= 0,
	EmaiImapBackward			= 1
};

/*!
// @brief	Retrieves previous or next numbers in sequence set.
//			The function is designed for using in conjunction with
//			EmaiImapSequenceSetGetBounds for iterating numbers stored in
//			sequence set object. Note, the function do not require the inNumber
//			parameter to be contained in sequence set,
//			EmaiImapSequenceSetGetNextNumber will return nearest number from
//			the sequense set.
//
//			Forward enumeration:
//
//			EmaiUint32 number;
//			EmaiError error = EmaiImapSequenceSetGetBounds(sequenceSet,
//									&number, NULL, EmaiNullOptions);
//			while (error == EmaiErrNoErr)
//			{
//				//perform operations with number
//				....
//				//retrieve next number from sequence set
//				error = EmaiImapSequenceSetGetNextNumber(sequenceSet, number,
//								&number, EmaiImapForward);
//			}
//			
//			Backward enumeration:
//
//			EmaiUint32 number;
//			EmaiError error = EmaiImapSequenceSetGetBounds(sequenceSet,
//									NULL, &number, EmaiNullOptions);
//			while (error == EmaiErrNoErr)
//			{
//				//perform operations with number
//				....
//				//retrieve next number from sequence set
//				error = EmaiImapSequenceSetGetNextNumber(sequenceSet, number,
//								&number, EmaiImapBackward);
//			}
//			
// @param	inSequenceSet - sequence set object to be examined
// @param	inNumber - current  number of the sequence set
// @param	outLastNumber - on exit contains next or previous (depending on
//						inOptions value) number of the sequence set
// @param	inOptions - EmaiImapForward or EmaiImapBackward (See
//						@ref EmaiEnumImapSequenceSetGetNextNumberOptions)
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceSetGetNextNumber(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32 inNumber,
		EmaiUint32* outNextNumber,
		EmaiOptions inOptions);

/*!
// @brief	Checks if number is contained in specified sequence set. The function
//			returns EmaiErrNoErr if number is contained in the sequence set or
//			EmaiErrIncorrectParameter otherwise.
//
// @param	inSequenceSet - sequence set object to be examined
// @param	inNumber - number to be examined
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiImapSequenceContainsNumber(
		EmaiImapSequenceSet inSequenceSet,
		EmaiUint32 inNumber,
		EmaiOptions inOptions);



/***************************************************
//
// IMAP server response structures: status responses
//
***************************************************/

/*!
// @brief	This structure is used for as for setting, so and for retrieving the
//			(mailbox / message) flags, passed to / from the server.
//			The strusture is used for both mailbox (for example: SELECT and EXAMINE
//			commands) and message related functions (for example: APPEND and STORE
//			commands).
//			The flag list identifies the applicable flags (at least the standard
//			defined flags, acording to RFC 2501). Custom flags can also exist,
//			depending of client implementation.
//
// @see		EmaiStatusResponse
// @see		EmaiResponseText
// @see		EmaiFlagsResponse
// @see		EmaiFetchResponse
// @see		EmaiStoreMessageData
*/
typedef struct
{
	/*! flags which are recognized by the library
	//  are added to the flags member*/
	EmaiUint32					flags;

	/*! flags which are not recognized by library
	//  are added to the customFlags*/
	EmaiStringList				customFlags;
} EmaiFlags;

/*!
// @brief	used for appenduid and copyuid response codes RFC4315
//
// @see		EmaiStatusResponse
*/
typedef struct
{
	/*! UIDVALIDITY of the destination mailbox*/
	EmaiUint32					uidValidity;

	/*! UID set containing the UIDs of the message(s) in the source mailbox,
	// it is set to NULL for APPENDUID response code*/
	EmaiImapSequenceSet			sourceSequenceSet;

	/*! UID assigned to the message(s) in the destination mailbox*/
	EmaiImapSequenceSet			destSequenceSet;
} EmaiUidPlus;


/*!
// @brief	These are known flags, which are used as a values of EmaiFlags and
//			EmaiFlagsResponse structures, for example:
//
// @see		EmaiFlags
// @see		EmaiFlagsResponse
// @see		EmaiResponse
// @see		EmaiStatusResponse
// @see		EmaiFetchResponse
// @see		EmaiStoreMessageData
// @see		EmaiImapAppendMessage
*/
enum EmaiEnumImapStatusResponseFlags
{
	EmaiAnsweredFlag			= 0x01,/*!< \\Answered \n
									   // Message has been answered*/

	EmaiFlaggedFlag				= 0x02,/*!< \\Flagged \n
									   // Message is "flagged" for urgent/special attention*/

	EmaiDeletedFlag				= 0x04,/*!< \\Deleted \n
									   // Message is "deleted" for removal by later EXPUNGE*/

	EmaiSeenFlag				= 0x08,/*!< \\Seen \n
									   // Message has been read*/

	EmaiDraftFlag				= 0x10,/*!< \\Draft \n
									   // Message has not completed composition (marked as a draft)*/
	
	EmaiAllowKeywordsFlag		= 0x20,/*!< \\* \n
									   // special flag , which indicates that it is possible to create new keywords
									   // by attempting to store those flags in the mailbox.\n
									   // This one can be present only as parameter of
									   // EmaiPermanentFlagsCode status code*/

	EmaiRecentFlag				= 0x20/*!< \\Recent \n
									  // Message has "recently" arrived in to this mailbox.  This session 
									  // is the first session to be notified about this message;
									  // if the session is read-write, subsequent sessions 
									  // will not see \\Recent set for this message.  This flag can not 
									  // be altered by the client.\n
									  // This flag can appear in FETCH response in
									  // FLAGS attributes list*/
};

/*!
// @brief	Status response codes. \n
//			Comments after constants show which member in the param
//			union of EmaiResponseText structure is filled for the corresponding
//			code value.
//
// @see		EmaiResponseText
*/
enum EmaiEnumImapStatusResponseCodes
{
	/* code is absent in the response text */
	EmaiNoCode,									/*!< none*/
	/* code is not recognized by library, check codeText value */
	EmaiXUserDefinedCode,						/*!< text*/

	/*Refer to chapter 77.1 of the RFC 3501 for more information regarding
	response codes*/
	EmaiAlertCode,								/*!< none*/
	EmaiBadCharsetCode,							/*!< list*/
	EmaiCapabilityCode,							/*!< list*/
	EmaiParseCode,								/*!< none*/
	EmaiPermanentFlagsCode,						/*!< permanentFlags*/
	EmaiReadOnlyCode,							/*!< none*/
	EmaiReadWriteCode,							/*!< none*/
	EmaiTryCreateCode,							/*!< none*/
	EmaiUIDNextCode,							/*!< number*/
	EmaiUIDValidityCode,						/*!< number*/
	EmaiUnseenCode,								/*!< number*/
	EmaiAppendUIDCode,							/*!< uidPlus*/
	EmaiCopyUIDCode								/*!< uidPlus*/
};

/*!
// @brief	EmaiResponseText structure.\n
//			This structure contains optional response codes and its parameters.
//
// @see		EmaiEnumImapStatusResponseCodes
// @see		EmaiStatusResponse
// @see		EmaiContinueRequestResponse
*/
typedef struct
{
	/*! Result code, for example: EmaiNoCode, etc*/
	EmaiUint32					code;

	/*! Textual representation of the code.*/
	EmaiConstUniCharPtr			codeText;
	
	/*! Parsed code parameters.*/
	union
	{
		EmaiConstUniCharPtr		text;
		EmaiUint32				number;
		EmaiUidPlus				uidPlus;
		EmaiFlags				permanentFlags;
		EmaiStringList			list;
	}							param;

	/* Text which follows optional response code.*/
	EmaiConstUniCharPtr			text;
} EmaiResponseText;


/*!
// @brief	Status response types
//
// @see		EmaiEnumImapServerResponses
// @see		EmaiStatusResponse
*/
enum EmaiEnumImapStatusResponseTypes
{
	EmaiImapOkResponse,						/*!< EmaiImapOkResponse*/
	EmaiImapNoResponse,						/*!< EmaiImapNoResponse*/
	EmaiImapBadResponse,					/*!< EmaiImapBadResponse*/
	EmaiImapPreauthResponse,				/*!< EmaiImapPreauthResponse*/
	EmaiImapByeResponse,					/*!< EmaiImapByeResponse*/
	
	EmaiTaggedStatusResponse	= 0x80000000,
											/*!< EmaiTaggedStatusResponse*/
	
	EmaiImapTaggedOkResponse	= EmaiTaggedStatusResponse | EmaiImapOkResponse
											/*!< EmaiImapTaggedOkResponse*/
};

/*!
// @brief	EmaiStatusResponse structure is used as a part
//			of EmaiResponse structure, and carries the actual
//			IMAP response code and explanation text.
//
// @see		EmaiEnumImapStatusResponseTypes
// @see		EmaiResponse
// @see		EmaiResponseText
*/
typedef struct
{
	/*! IMAP response code, for example: EmaiImapOkResponse, etc*/
	EmaiUint32					responseType;

	/*! IMAP response text, supplied as a comment to the responseType*/
	EmaiResponseText			responseText;
} EmaiStatusResponse;


/***************************************************
//
// IMAP server response structures: server and mailbox status
//
***************************************************/

/*!
// @brief	This structure is used as a part of EmaiResponse
//			structure, and carries the actual response returned
//			by the server for the CAPABILITY command.
//
// @see		EmaiResponse
*/
typedef struct
{
	EmaiStringList				capabilities;
} EmaiCapabilityResponse;

/*!
// @brief	Mailbox attributes. Four name attributes are defined:
//
//			\\Noinferiors
//			It is not possible for any child levels of hierarchy to exist
//			under this name; no child levels exist now and none can be
//			created in the future.\n
//
//			\\Noselect
//			It is not possible to use this name as a selectable mailbox.\n
//
//			\\Marked
//			The mailbox has been marked "interesting" by the server; the
//			mailbox probably contains messages that have been added since
//			the last time the mailbox was selected.\n
//
//			\\Unmarked
//			The mailbox does not contain any additional messages since the
//			last time the mailbox was selected.\n
//
//			If it is not feasible for the server to determine whether or not
//			the mailbox is "interesting", or if the name is a \\Noselect name,
//			the server SHOULD NOT send either \\Marked or \\Unmarked.\n
//
//			Refer to RFC3501 (chapter 7.2.2) for more information
//
//			The CHILDREN extension defines two new attributes that MAY be
//			returned within a LIST response.
//
//			\\HasChildren - The presence of this attribute indicates that the
//			mailbox has child mailboxes.
//
//			\\HasNoChildren - The presence of this attribute indicates that the
//			mailbox has NO child mailboxes that are accessible to the currently
//			authenticated user.  If a mailbox has the \\Noinferiors attribute, the
//			\\HasNoChildren attribute is redundant and SHOULD be omitted in the
//			LIST response.
//
//			Refer to RFC3348 for more information
//
// @see		EmaiListResponse
*/
enum EmaiEnumImapMailboxAttributes
{
	EmaiNoAttributes				= 0x0000,	/*!< EmaiNoselectAttribute*/

	/*The following three flags are exclusive*/
	EmaiNoselectAttribute			= 0x0001,	/*!< EmaiNoselectAttribute*/
	EmaiMarkedAttribute				= 0x0002,	/*!< EmaiMarkedAttribute*/
	EmaiUnmarkedAttribute			= 0x0003,	/*!< EmaiUnmarkedAttribute*/
	EmaiStandardAttibutesMask		= 0x0003,

	EmaiNoinferiorsAttribute		= 0x0010,	/*!< EmaiNoinferiorsAttribute*/

	/*CHILDREN extension according to RFC3348*/
	/*The following two flags are exclusive*/
	EmaiHasChildren					= 0x0100,
	EmaiHasNoChildren				= 0x0200,
	EmaiChildrenAttibutesMask		= 0x0300
};

/*!
// @brief	This response is returned by server for LIST and LSUB commands.
//			\n
//			LIST command returns a single name that matches the LIST specification.
//			There can be multiple LIST responses for a single LIST command.
//			The name represents an unambiguous left-to-right hierarchy, and
//			MUST be valid for use as a reference in LIST and LSUB commands.
//			Unless \Noselect is indicated, the name MUST also be valid as an
//			argument for commands, such as SELECT, that accept mailbox names.
//			\n
//			LSUB command returns a single name that matches the LSUB specification.
//			There can be multiple LSUB responses for a single LSUB command. The
//			data is identical in format to the LIST response.
//
// @see		Emai::ImapSession::ListMailboxes
// @see		EmaiImapListMailboxes
*/
typedef struct
{
	/*! Mailbox attribute, for example: EmaiNoselectAttribute etc*/
	EmaiUint32					attributes;

	/*! List of attributes which are not recognized by library*/
	EmaiStringList				customAttributes;

	/*! Hierarchy delimiter character (0 if name is "flat")*/
	EmaiChar					delimiter;

	/*! Mailbox name*/
	EmaiConstUniCharPtr			mailboxName;
} EmaiListResponse;

typedef EmaiListResponse		EmaiLsubResponse;

/*!
// @brief	Special value EmaiMailboxStatusResponse members. 
//
// @see		EmaiMailboxStatusResponse
*/
enum EmaiEnumImapUndefinedStatus
{
	EmaiUndefinedStatus			= 0xFFFFFFFF	/*!< EmaiUndefinedStatus*/
};

/*!
// @brief	The STATUS response occurs as a result of the STATUS command.  It 
//			returns the mailbox name that matches the STATUS specification and 
//			the requested mailbox status information.
//
// @see		Emai::ImapSession::MailboxStatus
// @see		EmaiImapMailboxStatus
// @see		EmaiEnumImapUndefinedStatus
*/
typedef struct
{
	EmaiConstUniCharPtr			mailboxName;
	
	/*!
	// The following five fields will get EmaiUndefinedStatus value if 
	// corresponding information is not present in response
	// \n\n
	*/

	/*! The number of messages in the mailbox*/
	EmaiUint32					messages;

	/*! The number of messages with the \\Recent flag set*/
	EmaiUint32					recent;

	/*! The next unique identifier value of the mailbox*/
	EmaiUint32					uidNext;

	/*! The unique identifier validity value of the mailbox*/
	EmaiUint32					uidValidity;

	/*! The number of messages which do not have the \\Seen flag set*/
	EmaiUint32					unseen;
} EmaiMailboxStatusResponse;

/*!
// @brief	This response is returned by server for SEARCH and UID SEARCH commands.
//			The number(s) refer to those messages that match the
//			search criteria. For SEARCH, these are message sequence numbers;
//			for UID SEARCH, these are unique identifiers.
//
// @see		Emai::ImapSession::Search
// @see		EmaiImapSearch
*/
typedef struct
{
	/*! Sequence set of the found message sequence numbers or UIDs*/
	EmaiImapSequenceSet			messages;
} EmaiSearchResponse;

/*!
// @brief	This response is returned by server for SELECT and EXAMINE commands.\n
//			The flag list identifies the flags (at least the system defined flags),
//			that are applicable for this mailbox. Flags other then system flags
//			can also exist, depending of server implementation.
//
// @see		Emai::ImapSession::SelectMailbox
// @see		EmaiImapSelectMailbox
// @see		Emai::ImapSession::ExamineMailbox
// @see		EmaiImapExamineMailbox
*/
/*! Combination of EmaiAnsweredFlag, EmaiFlaggedFlag, etc*/
typedef EmaiFlags				EmaiFlagsResponse;


/***************************************************
//
// IMAP server response structures: mailbox content
//
***************************************************/

/*!
// @brief	EmaiNumberResponse is a structure common for several response
//			types, such as EmaiExistsResponse, EmaiRecentResponse and
//			EmaiExpungeResponse.
//
*/
typedef struct
{
	/*! Depending of the response type the number which represents number of existing
	// recent messages, or expunged message sequence number*/
	EmaiUint32					number;
} EmaiNumberResponse;

/*!
// @brief	This response is returned by server for EXISTS command.
//			The EXISTS response reports the number of messages in the mailbox.
//			This response occurs as a result of a SELECT or EXAMINE command,
//			and if the size of the mailbox changes (e.g., new messages).
//			The update from the EXISTS response MUST be recorded by the
//			client.
//
// @see		Emai::ImapSession::SelectMailbox
// @see		EmaiImapSelectMailbox
// @see		Emai::ImapSession::ExamineMailbox
// @see		EmaiImapExamineMailbox
*/
typedef EmaiNumberResponse				EmaiExistsResponse;

/*!
// @brief	The RECENT response reports the number of messages with the
//			\\Recent flag set. This response occurs as a result of a SELECT or
//			EXAMINE command, and if the size of the mailbox changes (e.g., new
//			messages).
//
//			Note: It is not guaranteed that the message sequence
//			numbers of recent messages will be a contiguous range of
//			the highest n messages in the mailbox (where n is the
//			value reported by the RECENT response). Examples of
//			situations in which this is not the case are: multiple
//			clients having the same mailbox open (the first session
//			to be notified will see it as recent, others will
//			probably see it as non-recent), and when the mailbox is
//			re-ordered by a non-IMAP agent.
//			The only reliable way to identify recent messages is to
//			look at message flags to see which have the \\Recent flag
//			set, or to do a SEARCH RECENT.
//
//			The update from the RECENT response MUST be recorded by the
//			client.
//
// @see		Emai::ImapSession::SelectMailbox
// @see		EmaiImapSelectMailbox
// @see		Emai::ImapSession::ExamineMailbox
// @see		EmaiImapExamineMailbox
*/
typedef EmaiNumberResponse				EmaiRecentResponse;


/***************************************************
//
// IMAP server response structures: message status
//
***************************************************/

/*!
// @brief	This response can be returned by server for most of the commands. \n
//			(see 7.4.1 section of the RFC 3501) \n
//
//			The EXPUNGE response reports that the specified message sequence
//			number has been permanently removed from the mailbox. The message
//			sequence number for each successive message in the mailbox is
//			immediately decremented by 1, and this decrement is reflected in
//			message sequence numbers in subsequent responses (including other
//			untagged EXPUNGE responses).
//			The EXPUNGE response also decrements the number of messages in the
//			mailbox; it is not necessary to send an EXISTS response with the
//			new value.
//			As a result of the immediate decrement rule, message sequence
//			numbers that appear in a set of successive EXPUNGE responses
//			depend upon whether the messages are removed starting from lower
//			numbers to higher numbers, or from higher numbers to lower
//			numbers. For example, if the last 5 messages in a 9-message
//			mailbox are expunged, a "lower to higher" server will send five
//			untagged EXPUNGE responses for message sequence number 5, whereas
//			a "higher to lower server" will send successive untagged EXPUNGE
//			responses for message sequence numbers 9, 8, 7, 6, and 5.
//			An EXPUNGE response MUST NOT be sent when no command is in
//			progress, nor while responding to a FETCH, STORE, or SEARCH
//			command. This rule is necessary to prevent a loss of
//			synchronization of message sequence numbers between client and
//			server. A command is not "in progress" until the complete command
//			has been received; in particular, a command is not "in progress"
//			during the negotiation of command continuation.\n
//
//			Note: UID FETCH, UID STORE, and UID SEARCH are different
//			commands from FETCH, STORE, and SEARCH. An EXPUNGE
//			response MAY be sent during a UID command.\n
//
//			The update from the EXPUNGE response MUST be recorded by the
//			client.
//
//
// @see		EmaiResponse
*/
typedef EmaiNumberResponse				EmaiExpungeResponse;

/*!
// @brief	Special value EmaiBodySectionData dataOrigin member.
//
// @see		EmaiBodySectionData
*/
enum EmaiEnumImapUndefinedOriginValue
{
	EmaiUndefinedOriginValue			= 0xFFFFFFFF
};

/*!
// @brief	EmaiBodySectionData is used as a member of EmaiFetchResponse structure,
//			which is returned by the server for FETCH command.\n
//
//			The FETCH response returns data about a message to the client.
//			The data are pairs of data item names and their values in
//			parentheses. This response occurs as the result of a FETCH or
//			STORE command, as well as by unilateral server decision (e.g.,
//			flag updates).\n
//
//			For details see the RFC 3501 section 7.4.2.
//
// @see		EmaiFetchResponse
// @see		EmaiEnumImapUndefinedOriginValue
*/
typedef struct
{
	/*! section name*/
	EmaiConstUniCharPtr			section;

	/*! section origin, it is set to EmaiUndefinedOriginValue if absent in response*/
	EmaiUint32					dataOrigin;


	/*! section data*/
	EmaiConstCharPtr			data;

	/*! section data length*/
	EmaiUint32					dataLength;
} EmaiBodySectionData;

/*!
// @brief	EmaiRFC822Data is used as a member of EmaiFetchResponse structure,
//			which is returned by the server for FETCH and STORE commands.
//			If "data" member is NULL, this means that attribute is not present
//			in server response.
//
// @see		EmaiFetchResponse
*/
typedef struct
{
	/*! RFC822 data*/
	EmaiConstCharPtr			data;

	/*! RFC822 data length (in bytes)*/
	EmaiUint32					dataLength;
} EmaiRFC822Data;

/*!
// @brief	This structure represents the major message fields; it is used
//			as a part of the EmaiBodyStructure and EmaiFetchResponse structures.
//			Members of the structure are set to NULL if corresponding field is
//			not returned by server (most probably it is absent in message)
//
// @see		EmaiBodyStructure
// @see		EmaiFetchResponse
*/
typedef struct
{
	EmaiConstUniCharPtr			date;
	EmaiConstUniCharPtr			subject;

	EmaiContactList				from;
	EmaiContactList				sender;
	EmaiContactList				replyTo;
	EmaiContactList				to;
	EmaiContactList				cc;
	EmaiContactList				bcc;
	EmaiConstUniCharPtr			inReplyTo;
	
	EmaiConstUniCharPtr			messageID;
	
} EmaiFetchEnvelopeData;


/*!
// @brief	Message body type.
//
// @see		EmaiBodyStructure
*/
enum EmaiEnumBodyType
{
	EmaiMultipartBody,
	EmaiBasicBody,
	EmaiMessageBody,
	EmaiTextBody
};

/*!
// @brief	EmaiBodyStructure
//
// @see		EmaiEnumBodyType
*/
typedef struct EmaiBodyStructure
{
	/*! EmaiMultipartBody, EmaiBasicBody, etc
		@see EmaiEnumBodyType*/
	EmaiUint32					bodyType;

	/*! For multipart bodies the field contains
	parameters only for BODYSTRUCTURE fetch*/
	EmaiContentField			contentType;

	/*! this field can be non-NULL only for BODYSTRUCTURE fetch*/
	EmaiContentField			contentDisposition;

	/*! this field can be non-NULL only for BODYSTRUCTURE fetch*/
	EmaiStringList				languages;

	/*! this field can be non-NULL only for BODYSTRUCTURE fetch*/
	EmaiConstCharPtr			locationURI;

	/*! contentMD5 can be non-NULL only for BODYSTRUCTURE fetch
	and non-multipart parts*/
	EmaiConstCharPtr			contentMD5;

	/*! fields can be filled only for non-multipart parts, for
	multipart they set 0*/
	EmaiUint32					size;
	EmaiConstUniCharPtr			contentID;
	EmaiConstUniCharPtr			contentDescription;
	EmaiConstUniCharPtr			contentTransferEncoding;

	union
	{
		struct
		{
			EmaiUint32				subPartCount;
			/*! array with subPartCount elements*/
			const struct EmaiBodyStructure**
									subParts;
		}						multipart;

		struct
		{
			/*no additional data*/
		}						basic;
		
		struct
		{
			EmaiUint32				lines;
		}						text;
		
		struct
		{
			EmaiFetchEnvelopeData	envelope;
			EmaiUint32				lines;
			const struct EmaiBodyStructure*
									body;
		}						message;
		
	} data;
	
} EmaiBodyStructure;

/*!
// @brief	This response is returned by server for FETCH and STORE commands.
//
// @see		Emai::ImapSession::Fetch
// @see		EmaiImapFetch
*/
typedef struct
{
	/*! message sequence number attribute*/
	EmaiUint32					messageNumber;

	/*! combination of EmaiAnsweredFlag, EmaiFlaggedFlag etc*/
	EmaiFlags					flags;
	
	/*! envelope data for the message*/
	EmaiFetchEnvelopeData		envelope;
	
	/*! message internal date*/
	EmaiConstUniCharPtr			internalDate;
	
	/*! UID message attribute*/
	EmaiUint32					uid;

	/*! RFC822.SIZE message attribute*/
	EmaiUint32					rfc822Size;
	
	/*! RFC822 message attribute*/
	EmaiRFC822Data				rfc822;
	/*! RFC822.HEADER message attribute*/
	EmaiRFC822Data				rfc822Header;
	/*! RFC822.TEXT message attribute*/
	EmaiRFC822Data				rfc822Text;
	
	/*! BODY[<section>]<<origin octet>> fetch data*/
	struct
	{
		EmaiUint32					count;
		/*! array with count elements*/
		const EmaiBodySectionData*	data;
	}							sections;
	
	/*! BODY fetch value, NULL if not present*/
	const EmaiBodyStructure*	body;
	
	/*! BODYSTRUCTURE fetch value, NULL if not present*/
	const EmaiBodyStructure*	bodyStructure;

} EmaiFetchResponse;


/***************************************************
//
// IMAP server response structures: continue request
//
***************************************************/

/*!
// @brief	EmaiContinueRequestResponse
//
// @see		EmaiResponseText
*/
typedef struct
{
	/*! IMAP response text, supplied as a comment to continue request*/
	EmaiResponseText			responseText;
} EmaiContinueRequestResponse;

/***************************************************
//
// IMAP server response structures: response data
//
***************************************************/

/*!
// @brief	IMAP server response structures: response data
//			Comments after constants show which member in the data
//			union of EmaiResponse structure is filled for the corresponding
//			responseCode value.
//
// @see		EmaiEnumImapStatusResponseTypes
*/
enum EmaiEnumImapServerResponses
{
	EmaiUnrecognizedResponseType,				/*!< none*/
	EmaiStatusResponseType,						/*!< status*/
	EmaiCapabilityResponseType,					/*!< capability*/
	EmaiListResponseType,						/*!< list*/
	EmaiLsubResponseType,						/*!< lsub*/
	EmaiMailboxStatusResponseType,				/*!< mailboxStatus*/
	EmaiSearchResponseType,						/*!< search*/
	EmaiFlagsResponseType,						/*!< flags*/
	EmaiExistsResponseType,						/*!< exists*/
	EmaiRecentResponseType,						/*!< recent*/
	EmaiExpungeResponseType,					/*!< expunge*/
	EmaiFetchResponseType,						/*!< fetch*/
	EmaiContinueRequestResponseType				/*!< continueRequest*/
};

/*!
// @brief	EmaiResponse structure is used to return the server reponse.
//
// @see		EmaiEnumImapStatusResponseTypes
// @see		EmaiImapGetRespose
*/
typedef struct
{
	/*! Version of the structure,*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	/*! pointer on server response data*/
	EmaiConstCharPtr			response;
	/*! size of the server response data*/
	EmaiUint32					responseSize;
	/*! EmaiStatusResponseType etc*/
	EmaiUint32					responseType;
	
	/*! response data, the real*/
	union
	{
		EmaiStatusResponse			status;
		EmaiCapabilityResponse		capability;
		EmaiListResponse			list;
		EmaiLsubResponse			lsub;
		EmaiMailboxStatusResponse	mailboxStatus;
		EmaiSearchResponse			search;
		EmaiFlagsResponse			flags;
		EmaiExistsResponse			exists;
		EmaiRecentResponse			recent;
		EmaiExpungeResponse			expunge;
		EmaiFetchResponse			fetch;
		EmaiContinueRequestResponse	continueRequest;
	}							data;
} EmaiResponse;


/***************************************************
//
// Function for communicating with IMAP server
//
***************************************************/

/*!
// @brief	Values for selector member of EmaiImapCallbackData
//
// @see		EmaiImapCallbackData
*/
enum EmaiEnumImapCallbackData
{
	EmaiImapSelectorNormal						= 0,
	EmaiImapSelectorSending,
	EmaiImapSelectorReceiving,
	EmaiImapSelectorServerError,
};

/*!
// @brief	Constants for operation codes for smtp sessions
//
// @see		EmaiImapCallbackData
*/
enum EmaiEnumImapOperationCode
{
	EmaiImapConnectingToServer		= 0,
	EmaiImapConnectedToServer,
	EmaiImapAuthenticating,
	EmaiImapAuthenticated,
	EmaiImapCreatingMailbox,
	EmaiImapMailboxCreated,
	EmaiImapDeletingMailbox,
	EmaiImapMailboxDeleted,
	EmaiImapSubsribingMailbox,
	EmaiImapMailboxSubscribed,
	EmaiImapUnSubsribingMailbox,
	EmaiImapMailboxUnSubscribed,
	EmaiImapRenamingMailbox,
	EmaiImapMailboxRenamed,
	EmaiImapRetrievingMailboxList,
	EmaiImapMailboxListRetrieved,
	EmaiImapExaminingMailbox,
	EmaiImapMailboxExamined,
	EmaiImapRetrievingMailboxStatus,
	EmaiImapMailboxStatusRetrieved,
	EmaiImapSelectingMailbox,
	EmaiImapMailboxSelected,
	EmaiImapClosingMailbox,
	EmaiImapMailboxClosed,
	EmaiImapChecking,
	EmaiImapChecked,
	EmaiImapAppendingMessage,
	EmaiImapMessageAppended,
	EmaiImapSearchingMessages,
	EmaiImapMessageSearchFinished,
	EmaiImapFetchingMessages,
	EmaiImapMessagesFetched,
	EmaiImapCopyingMessages,
	EmaiImapMessagesCopied,
	EmaiImapStoringMessagesData,
	EmaiImapMessagesDataStored,
	EmaiImapExpungingMessages,
	EmaiImapMessagesExpunged,
	EmaiImapSendingNOOP,
	EmaiImapNOOPSent,
	EmaiImapClosingConnection,
	EmaiImapConnectionClosed
};


/*!
// @brief	EmaiImapCallbackData
//
// @see		EMAICALLBACKAPI
*/
typedef struct
{
	/*! Version of the structure,*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	EmaiUint32					selector;
	union
	{
		/*! selector == EmaiImapSelectorNormal*/
		struct
		{
			EmaiUint32			operationCode;
		} normal;

		/*! selector == EmaiImapSelectorSending*/
		struct
		{
			EmaiUint32			sentSize;
			EmaiUint32			totalSize;
		} sending;

		/*! selector == EmaiImapSelectorReceiving*/
		struct
		{
			EmaiUint32			receivedSize;
			EmaiUint32			totalSize;
		} receiving;

		/*! selector == EmaiImapSelectorServerError*/
		struct
		{
			EmaiConstCharPtr	serverReply;
			EmaiUint32			replyCode;
		} serverError;
	} data;
} EmaiImapCallbackData;

/*!
// @brief	Status callback declaration for IMAP session
*/
typedef EMAICALLBACKAPI(EmaiError, EmaiImapStatusCallback)(
		EmaiImapSession inSession,
		const EmaiImapCallbackData* inData,
		void* inUserData);

/*!
// @brief	EmaiImapCommandData
//
// @see		EmaiImapResponseCallback
*/
typedef struct
{
	/*! Version of the structure,*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	/*! points on selected mailbox for all selected state commands, for
	authentication state commands SELECT, EXAMINE, DELETE, RENAME and
	STATUS it points on mailbox name which is specified as command parameter,
	for other commands it set to NULL */
	EmaiConstUniCharPtr			sourceMailboxName;

	/*! points on destination mailbox name for APPEND and COPY commands, points
	on new mailbox name for RENAME and CREATE commands and to NULL for other commands*/
	EmaiConstUniCharPtr			destMailboxName;

	/*! contains number of message specified for APPEND command*/
	EmaiUint32					appendedMessageCount;

	/*! pointer to the array of appendedMessageCount message objects specified for
	APPEND command*/
	const EmaiMessage*			appendedMessages;
	
} EmaiImapCommandData;

/*!
// @brief	Response handling callback declaration for IMAP session
//
// @param	inSession - session object, for which callback was triggered
// @param	inCommandData - pointer to the EmaiImapCommandData structure, which
//						contains useful information about current command
// @param	inResponse - parsed response received from server
// @param	inUserData - user data specified in inResponseUserData parameter of
//						EmaiImapSessionCreate function
//
// @see		EmaiImapSessionCreate
// @see		EmaiImapCommandData
// @see		EmaiResponse
//
*/
typedef EMAICALLBACKAPI(void, EmaiImapResponseCallback)(
		EmaiImapSession inSession,
		const EmaiImapCommandData* inCommandData,
		const EmaiResponse* inResponse,
		void* inUserData);


/*!
// @brief	Options for EmaiImapSessionCreate function
//
// @see		EmaiImapSessionCreate
*/
enum EmaiEnumImapUseSSLOption
{
	/*! This session will connect only to secured IMAP servers*/
	EmaiImapUseSSLOption					= 0x01
};

/*!
// @brief	Creates IMAP session object
//
// @param	outSession - on exit contains session object
// @param	inResponseCallback - response callback
// @param	inResponseUserData - user data which is passed in response callback
//						as inUserData parameter
// @param	inStatusCallback - status callback
// @param	inStatusUserData - user data which is passed in status callback
//						as inUserData parameter
// @param	inTimeout - timeout value in seconds
// @param	inOptions - use EmaiUseSSLOption to create secured
//						SSL session or EmaiNullOptions otherwise
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapSessionCreate(
		EmaiImapSession* outSession,
		EmaiImapResponseCallback inResponseCallback,
		void* inResponseUserData,
		EmaiImapStatusCallback inStatusCallback,		/*can be NULL*/
		void* inStatusUserData,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions);


/*!
// @brief	Constants for default IMAP ports.
//
// @see		EmaiImapConnect
// @see		Emai::ImapSession::Connect
*/
enum EmaiEnumImapPorts
{
	EmaiDefaultImapPort					= 143,
	EmaiDefaultImapSSLPort				= 993
};

/*!
// @brief	Connects IMAP session object to the specified
//			mail server
//
// @param	inSession - session object to be connected
// @param	inMailServer - pointer to null-teminated string which
//						specifies name or IP address of IMAP server
// @param	inPort - port number of IMAP server
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiEnumImapPorts
// @see		Emai::ImapSession::EmaiImapConnect
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapConnect(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailServer,
		short inPort,
		EmaiOptions inOptions);

/*!
// @brief	Connects IMAP session object to the specified
//			mail server
//
// @param	inSession - session object to be connected
// @param	inMailServer - pointer to null-teminated string which
//						specifies name or IP address of IMAP server
// @param	inPort - port number of IMAP server
// @param	inProxyInfo - proxy server information struct
// @param	inUserData - user data specified parameter, use NULL
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiEnumImapPorts
// @see		Emai::ImapSession::EmaiImapConnectEx
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapConnectEx(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailServer,
		short inPort,
		const EmaiProxyInfo* inProxyInfo,
		void* inUserData,
		EmaiOptions inOptions);

/*!
// @brief	Disconnects IMAP session object from mail server
//
// @param	inSession - session object to be disconnected
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::EmaiImapDisconnect
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapDisconnect(
		EmaiImapSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Can be used as a periodic poll for new messages or message status updates
//			during a period of inactivity. \n
//
//			Refer to RFC3501 (chapter 6.4.1) for more information
//
// @param	inSession - session object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::Noop
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapNoop(
		EmaiImapSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Marks IMAP session to be cancelled ASAP
//
// @param	inSession - session object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::Cancel
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapCancel(
		EmaiImapSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Authentication modes for EmaiImapAuthenticate function
//
// @see		Emai::ImapSession::Authenticate
// @see		EmaiImapAuthenticate
*/
enum EmaiEnumImapAuthModes
{
	/*! Automatically tries all available mechanisms*/
	EmaiIMAPAuthAutomatic,
	
	/*! LOGIN authentification*/
	EmaiIMAPAuthLogin,

	/*! PLAIN authentification*/
	EmaiIMAPAuthPlain,
	
	/*! Cram-MD5 authentification*/
	EmaiIMAPAuthCramMD5,
	
	/*! NTLM authentification*/
	EmaiIMAPAuthNTLM,
	
	/*! LOGIN command authentification*/
	EmaiIMAPAuthLoginCommand
};

/*!
// @brief	Authenticates IMAP session
//
// @param	inSession - session object
// @param	inAccount - pointer to null-teminated string to specify
//						account name for authentication
// @param	inPassword - pointer to null-teminated string to specify
//						password for authentication
// @param	inOptions - Authentication method to be used. For example:
//						EmaiIMAPAuthAutomatic, EmaiIMAPAuthLogin, etc
//
//			Refer to RFC3501 (chapter 6.2.2) for more information.
//
// @see		Emai::ImapSession::Authenticate
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapAuthenticate(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inPassword,
		EmaiOptions inOptions);

/*!
// @brief	The SELECT command selects a mailbox so that messages in the
//			mailbox can be accessed.
//			Only one mailbox can be selected at a time in a connection;
//			simultaneous access to multiple mailboxes requires multiple
//			connections. The SELECT command automatically deselects any
//			currently selected mailbox before attempting the new selection.
//			Consequently, if a mailbox is selected and a SELECT command that
//			fails is attempted, no mailbox is selected.\n
//
//			Refer to RFC3501 (chapter 6.3.1) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::SelectMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapSelectMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The EXAMINE command is identical to SELECT and returns the same
//			output; however, the selected mailbox is identified as read-only.
//			No changes to the permanent state of the mailbox, including
//			per-user state, are permitted; in particular, EXAMINE MUST NOT
//			cause messages to lose the \\Recent flag.\n
//
//			Refer to RFC3501 (chapter 6.3.2) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::ExamineMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapExamineMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The CREATE command creates a mailbox with the given name. A positive
//			response is returned only if a new mailbox with that name has been
//			created. It is an error to attempt to create INBOX or a mailbox
//			with a name that refers to an extant mailbox. Any error in
//			creation will return an apropriate error code.\n
//
//			Refer to RFC3501 (chapter 6.3.3) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::CreateMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapCreateMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The DELETE command permanently removes the mailbox with the given
//			name. A positive response is returned only if the mailbox has
//			been deleted. It is an error to attempt to delete INBOX or a
//			mailbox name that does not exist.\n
//
//			Refer to RFC3501 (chapter 6.3.4) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::DeleteMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapDeleteMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The RENAME command changes the name of a mailbox. A positive
//			response is returned only if the mailbox has been renamed. It is
//			an error to attempt to rename from a mailbox name that does not
//			exist or to a mailbox name that already exists. Any error in
//			renaming will return an apropriate error code.\n
//
//			Renaming INBOX is permitted, and has special behavior. It moves
//			all messages in INBOX to a new mailbox with the given name,
//			leaving INBOX empty. If the server implementation supports
//			inferior hierarchical names of INBOX, these are unaffected by a
//			rename of INBOX.\n
//
//			Refer to RFC3501 (chapter 6.3.5) for more information.
//
// @param	inSession			- session object
// @param	inMailbox			- original mailbox to be renamed
// @param	inNewMailboxName	- new name for the mailbox
// @param	inOptions			- not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::RenameMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapRenameMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiConstUniCharPtr inNewMailboxName,
		EmaiOptions inOptions);

/*!
// @brief	The SUBSCRIBE command adds the specified mailbox name to the
//			server's set of "active" or "subscribed" mailboxes as returned by
//			the LSUB command. This command returns a positive response only
//			if the subscription is successful.\n
//
//			Refer to RFC3501 (chapter 6.3.6) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::SubscribeMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapSubscribeMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The UNSUBSCRIBE command removes the specified mailbox name from
//			the server's set of "active" or "subscribed" mailboxes as returned
//			by the LSUB command. This command returns a positive response
//			only if the unsubscription is successful.\n
//
//			Refer to RFC3501 (chapter 6.3.7) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::UnsubscribeMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapUnsubscribeMailbox(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The LIST command returns a subset of names from the complete set
//			of all names available to the client. Zero or more untagged LIST
//			replies are returned, containing the name attributes, hierarchy
//			delimiter, and name; see the description of the LIST reply for
//			more detail.\n
//
//			The character "*" is a wildcard, and matches zero or more
//			characters at this position. The character "%" is similar to "*",
//			but it does not match a hierarchy delimiter.\n
//
//			Refer to RFC3501 (chapter 6.3.8) for more information.
//
// @param	inSession - session object
// @param	inReferenceName - reference name
// @param	inWildcardedMailboxName - mailbox name with possible wildcards
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::ListMailboxes
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapListMailboxes(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inReferenceName,
		EmaiConstUniCharPtr inWildcardedMailboxName,
		EmaiOptions inOptions);

/*!
// @brief	The LSUB command returns a subset of names from the set of names
//			that the user has declared as being "active" or "subscribed".
//			Zero or more untagged LSUB replies are returned. The arguments to
//			LSUB are in the same form as those for LIST.\n
//
//			Refer to RFC3501 (chapter 6.3.9) for more information.
//
// @param	inSession - session object
// @param	inReferenceName - reference name
// @param	inWildcardedMailboxName - mailbox name with possible wildcards
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::ListSubscribedMailboxes
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapListSubscribedMailboxes(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inReferenceName,
		EmaiConstUniCharPtr inWildcardedMailboxName,
		EmaiOptions inOptions);

/*!
// @brief	Options to be used with EmaiImapMailboxStatus function. They define
//			the status data items that can be requested.
//
// @see		Emai::ImapSession::MailboxStatus
// @see		EmaiImapMailboxStatus
*/
enum EmaiEnumImapMailboxStatus
{
	/* The number of messages in the mailbox*/
	EmaiMessagesNumber						= 0x01,
	/* The number of messages with the \\Recent flag set*/
	EmaiRecentMessages						= 0x02,
	/* The next unique identifier value of the mailbox*/
	EmaiUidNext								= 0x04,
	/* The unique identifier validity value of the mailbox*/
	EmaiUidValidity							= 0x08,
	/* The number of messages which do not have the \\Seen flag set*/
	EmaiUnseenMessages						= 0x10,
	
	EmaiAllStatusData						= EmaiMessagesNumber | EmaiRecentMessages |
									EmaiUidNext | EmaiUidValidity | EmaiUnseenMessages
};

/*!
// @brief	The STATUS command requests the status of the indicated mailbox.
//			It does not change the currently selected mailbox, nor does it
//			affect the state of any messages in the queried mailbox (in
//			particular, STATUS MUST NOT cause messages to lose the \\Recent
//			flag).\n
//
//			The STATUS command provides an alternative to opening a second
//			IMAP4rev1 connection and doing an EXAMINE command on a mailbox to
//			query that mailbox's status without deselecting the current
//			mailbox in the first IMAP4rev1 connection.\n
//
//			Refer to RFC3501 (chapter 6.3.10) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inOptions - EmaiMessagesNumber, etc
//
// @see		Emai::ImapSession::MailboxStatus
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapMailboxStatus(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

/*!
// @brief	The APPEND command appends the new message
//			to the end of the specified destination mailbox. This argument
//			SHOULD be in the format of an [RFC-2822] message. 8-bit
//			characters are permitted in the message.\n
//
//			If the destination mailbox does not exist, a server MUST return an
//			error, and MUST NOT automatically create the mailbox.
//			If the mailbox is currently selected, the normal new message
//			actions SHOULD occur.\n
//
//			Refer to RFC3501 (chapter 6.3.11) for more information.
//
// @param	inSession - session object
// @param	inMailbox - mailbox name
// @param	inMessageFlags - zero terminated string with space delimited
//			message flags (for example: "\\Draft \\Flagged")
// @param	inDateTime - zero terminated date/time string, which indicates
//			internal message date (for example: " 1-Jan-2006 21:34:27 +0200")
// @param	inMessage - message object to be appended
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::AppendMessage
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapAppendMessage(
		EmaiImapSession inSession,
		EmaiConstUniCharPtr inMailbox,
		const EmaiFlags* inMessageFlags,	/*can be NULL*/
		EmaiConstUniCharPtr inDateTime,		/*can be NULL*/
		EmaiMessage inMessage,
		EmaiOptions inOptions);

/*!
// @brief	The CHECK command requests a checkpoint of the currently selected
//			mailbox. A checkpoint refers to any implementation-dependent
//			housekeeping associated with the mailbox (e.g., resolving the
//			server's in-memory state of the mailbox with the state on its
//			disk) that is not normally executed as part of each command. A
//			checkpoint MAY take a non-instantaneous amount of real time to
//			complete. If a server implementation has no such housekeeping
//			considerations, CHECK is equivalent to NOOP.\n
//
//			Refer to RFC3501 (chapter 6.4.1) for more information.
//
// @param	inSession - session object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::Check
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapCheck(
		EmaiImapSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Permanently removes all messages that have \\Deleted flag set from
//			the currently selected mailbox, and returns to the authenticated
//			state from the selected state.\n
//
//			Refer to RFC3501 (chapter 6.4.2) for more information.
//
// @param	inSession - session object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::ImapSession::CloseMailbox
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapCloseMailbox(
		EmaiImapSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	option for EmaiImapExpunge function.
//
// @see		Emai::ImapSession::Expunge
// @see		EmaiImapExpunge
*/
enum EmaiEnumImapExpungeOptions
{
	/*! This option can be used in EmaiImapExpunge function. It will cause*/
	/*! that skipping custom expunge algorithm if UID EXPUNGE command is not*/
	/*! supported by server. Caller can implement his own version of expunge*/
	/*! algorithm*/
	EmaiImapNoCustomExpungeOption					= 0x01,
};

/*!
// @brief	Permanently removes messages that have \\Deleted flag set from
//			the currently selected mailbox. \n
//			If inMessageUIDSequence parameter is NULL the function simply sends
//			EXPUNGE command on server. Otherwise it sends UID EXPUNGE command
//			(if it is supported by server) or perform following algorithm:
//				1) Find all messages marked \\Deleted on the server.
//				2) Find all messages that must not be erased.
//				3) Temporarily remove \\Deleted flag on all messages found in
//					step 2.
//				4) Expunge the mailbox.
//				5) Restore \\Deleted flag on all messages found when performing 
//					step 2.
//			Note: Its possible for another client to mark additional messages
//				as deleted while this sequence is being performed.  In this
//				case, these additional messages will be expunged as well.
//
//			The algorithm above can be skipped by passing 
//			EmaiImapNoCustomExpungeOption as inOption parameter value. In this
//			case no messages will be expunged. This allows caller to use only
//			UID EXPUNGE command if it supported by server, if UID EXPUNGE command
//			is not suppoted the EmaiErrCommandNotSupported error code returned
//			and nothing expunged from mailbox. Caller should implement his own
//			expunge algorithm in this case.
//
//			Refer to RFC3501 (chapter 6.4.3) for more information.
//			Refer to RFC4315 (chapter 2.1) for more information
//			Refer to RFC4549 (chapter 4.2.4) for more information
//
// @param	inSession - session object
// @param	inMessageUIDSequence - sequence of the message UIDs to be expunged
// @param	inOptions - EmaiImapNoCustomExpungeOption or EmaiNullOptions
//
// @see		Emai::ImapSession::Expunge
// @see		EmaiEnumImapExpungeOptions
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapExpunge(
		EmaiImapSession inSession,
		EmaiImapSequenceSet inMessageUIDSequence,	/*can be NULL*/
		EmaiOptions inOptions);

/*!
// @brief	UID option for COPY, FETCH, SEARCH and STORE commands.
//
// @see		Emai::ImapSession::Search
// @see		ImaiImapSearch
// @see		Emai::ImapSession::Fetch
// @see		EmaiImapFetch
// @see		Emai::ImapSession::Store
// @see		EmaiImapStore
// @see		Emai::ImapSession::Copy
// @see		EmaiImapCopy
*/
enum EmaiEnumImapUIDOptions
{
	/*! This option can be used in EmaiImapSearch, EmaiImapFetch,*/
	/*! EmaiImapStore and EmaiImapCopy functions. It will cause*/
	/*! that "UID *" command will be sent on server*/
	EmaiImapUIDOption					= 0x01,
};

/*!
// @brief	Values for selector fields of the EmaiSearchingKey structure.\n
//			Comments after constants show which members of the "param" union
//			should be filled for the corresponding selector value.
//
// @see		Emai::ImapSession::Search
// @see		ImaiImapSearch
*/
enum EmaiEnumImapSearchOptions
{
	/*! none*/
	EmaiSearchAll,
	EmaiSearchAnswered,
	EmaiSearchDeleted,
	EmaiSearchDraft,
	EmaiSearchFlagged,
	EmaiSearchNew,
	EmaiSearchOld,
	EmaiSearchRecent,
	EmaiSearchSeen,
	EmaiSearchUnanswered,
	EmaiSearchUndeleted,
	EmaiSearchUndraft,
	EmaiSearchUnflagged,
	EmaiSearchUnseen,

	/*! messageSequence*/
	EmaiSearchMessageSequence,
	EmaiSearchUid,

	/*! dateString*/
	EmaiSearchBefore,
	EmaiSearchOn,
	EmaiSearchSince,
	EmaiSearchSentBefore,
	EmaiSearchSentOn,
	EmaiSearchSentSince,

	/*! searchingString*/
	EmaiSearchBcc,
	EmaiSearchBody,
	EmaiSearchCc,
	EmaiSearchTo,
	EmaiSearchFrom,
	EmaiSearchKeyword,
	EmaiSearchSubject,
	EmaiSearchText,
	EmaiSearchUnkeyword,

	/*! headerField*/
	EmaiSearchHeader,

	/*! number*/
	EmaiSearchLarger,
	EmaiSearchSmaller,

	/*! searchingKey*/
	EmaiSearchNot,

	/*! seachingKeys*/
	EmaiSearchOr,

	/*! keyList*/
	EmaiSearchKeyList
};

/*!
// @brief	Search keys to be used with the EmaiSearchingCriteria structure,
//			and apropriate seach functions.
//
// @see		Emai::ImapSession::Search
// @see		ImaiImapSearch
*/
typedef struct EmaiSearchingKey
{
	/*! @see EmaiEnumImapSearchOptions*/
	EmaiUint32					selector;
	union
	{
		EmaiConstUniCharPtr		searchingString;

		EmaiImapSequenceSet		messageSequence;

		EmaiConstUniCharPtr		dateString;

		struct
		{
			EmaiConstUniCharPtr	name;
			EmaiConstUniCharPtr	searchingString;
		} headerField;
		
		EmaiUint32				number;
		
		/*! pointer on EmaiSearchingKey structure*/
		const struct EmaiSearchingKey*	searchingKey;
		
		/*! array with two elements*/
		const struct EmaiSearchingKey*	searchingKeys;

		struct
		{
			EmaiUint32				count;
			/*! array with count elements*/
			const struct EmaiSearchingKey*	searchingKeys;
		} keyList;
	} param;
} EmaiSearchingKey;

/*!
// @brief	Values for selector fields of the EmaiSearchingKey structure
//
// @see		Emai::ImapSession::Search
// @see		ImaiImapSearch
*/
enum EmaiEnumImapSearching
{
	EmaiSearchingCriteriaType,
	EmaiSearchingKeysType
};

/*!
// @brief	Search criterias to be used with the EmaiImapSearch and
//			Emai::ImapSession::Search functions.
//
// @see		Emai::ImapSession::Search
// @see		ImaiImapSearch
// @see		EmaiEnumMessageCharset
*/
typedef struct
{
	/*! version of the structure*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	/*! kEmaiUSASCII etc*/
	/*! specify kEmaiUnknown if character set should not appear in search*/
	EmaiUint32					characterSet;

	/*! EmaiSearchingCriteriaType or */
	/*! EmaiSearchingKeysType, selector for*/
	/*! the union below*/
	EmaiUint32					selector;		
	
	union
	{
		/*selector == EmaiSearchingCriteriaType*/
		EmaiConstUniCharPtr		searchingCriteria;

		/*selector == EmaiSearchingKeysType*/
		struct
		{
			EmaiUint32					count;
			/*! array with count elements*/
			const EmaiSearchingKey*		keys;
		}						searchingKeys;
	} data;
} EmaiSearchingCriteria;

/*!
// @brief	The SEARCH command searches the mailbox for messages that match
//			the given searching criteria. Searching criteria consist of one
//			or more search keys.\n
//
//			When multiple keys are specified, the result is the intersection
//			(AND function) of all the messages that match those keys. For
//			example, the criteria DELETED FROM "SMITH" SINCE 1-Feb-1994 refers
//			to all deleted messages from Smith that were placed in the mailbox
//			since February 1, 1994. A search key can also be a parenthesized
//			list of one or more search keys (e.g., for use with the OR and NOT
//			keys).\n
//
//			Refer to RFC3501 (chapter 6.4.4) for more information.
//
// @param	inSession - session object
// @param	inSearchingCriteria - if NULL all messages in mailbox are found
// @param	inOptions - specify EmaiImapUIDOption if you want to receive message UIDs
//						instead of message numbers in SEARCH response
//
// @see		Emai::ImapSession::Search
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapSearch(
		EmaiImapSession inSession,
		const EmaiSearchingCriteria* inSearchingCriteria,/*can be NULL*/
		EmaiOptions inOptions);

/*!
// @brief	Values for selector field of the EmaiFetchMessageAttribute structure.\n
//			Comments near constants shows which members in the "param" union
//			should be filled for the corresponding selector value.
//
// @see		Emai::ImapSession::Fetch
// @see		EmaiImapFetch
*/
enum EmaiEnumImapFetchAttribute
{
	/*! none*/
	EmaiFetchEnvelope,
	EmaiFetchFlags,
	EmaiFetchInternalDate,
	EmaiFetchRFC822,
	EmaiFetchRFC822Header,
	EmaiFetchRFC822Size,
	EmaiFetchRFC822Text,
	EmaiFetchBody,
	EmaiFetchBodyStructure,
	EmaiFetchUid,

	/*! data*/
	EmaiFetchBodySection,

	/*! data*/
	EmaiFetchBodyPeek
};

/*!
// @brief	Attributes for EmaiFetchMessageData structure
//
// @see		EmaiFetchMessageData
*/
typedef struct
{
	/*! Values: EmaiFetchEnvelope, EmaiFetchFlags, etc*/
	EmaiUint32					selector;
	union
	{
		/*!
		// Should be filled when performing BODY[<section>]<<partial>> or
		// BODY.PEEK[<section>]<<partial>> fetch, i.e when
		// (selector == EmaiFetchBodySection) or (selector == EmaiFetchBodyPeek)
		// Refer to RFC3501 (chapter 6.4.5) for more information
		*/
		struct
		{
			/*! section specification*/
			EmaiConstUniCharPtr	section;
			/*! partial fetch specification*/
			EmaiConstUniCharPtr	partial;
		} data;
	} param;
} EmaiFetchMessageAttribute;

/*!
// @brief	Values for selector field of the EmaiFetchMessageData structure.\n
//			Comments near constants shows which members in the "param" union
//			should be filled for the corresponding selector value.
//
// @see		Emai::ImapSession::Fetch
// @see		EmaiImapFetch
*/
enum EmaiEnumImapDataFetchOptions
{
	/*! none*/
	EmaiDataFetchAll,

	/*! none*/
	EmaiDataFetchFull,

	/*! none*/
	EmaiDataFetchFast,

	/*! attr*/
	EmaiDataFetchAttr,

	/*! data*/
	EmaiDataFetchString
};

/*!
// @brief	This structure identifies the message data to be fetched
//			using the EmaiImapFetch or Emai::ImapSession::Fetch functions.
//
// @see		Emai::ImapSession::Fetch
// @see		EmaiImapFetch
*/
typedef struct
{
	/*! version of the structure*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	/*! Values: EmaiDataFetchAll, EmaiDataFetchFull, etc*/
	EmaiUint32					selector;
	union
	{
		struct
		{
			EmaiUint32					attributeCount;
			const EmaiFetchMessageAttribute*
										attributes;
		}						attr;
		
		EmaiConstUniCharPtr		data;
	} param;
} EmaiFetchMessageData;

/*!
// @brief	The FETCH command retrieves data associated with a message in the
//			mailbox. The data items to be fetched can be either a single atom
//			or a parenthesized list.\n
//
//			Refer to RFC3501 (chapter 6.4.5) for more information.
//
// @param	inSession - session object
// @param	inMessageSequence - sequence of the message numbers or message UIDs
						to be fetched
// @param	inMessageData - message data to be retrieved
// @param	inOptions - if EmaiImapUIDOption is specified inMessageSequence
//						should specify message UIDs sequense instead of
//						message numbers sequense
//
// @see		Emai::ImapSession::Fetch
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapFetch(
		EmaiImapSession inSession,
		EmaiImapSequenceSet inMessageSequence,
		const EmaiFetchMessageData* inMessageData,
		EmaiOptions inOptions);

/*!
// @brief	Options for EmaiStoreMessageData structure
//
// @see		Emai::ImapSession::Store
// @see		EmaiImapStore
*/
enum EmaiEnumImapStoreOptions
{
	/*! The following three flags are exclusive*/
	EmaiStoreSetFlags,
	EmaiStoreAddFlags,
	EmaiStoreRemoveFlags,
	
	EmaiStoreSilent				= 0x10
};

/*!
// @brief	Values for selector field of the EmaiFetchMessageData structure.\n
//
//			Comments after constants show which members of the "param" union
//			should be filled for the corresponding selector value.
//
// @see		Emai::ImapSession::Store
// @see		EmaiImapStore
*/
enum EmaiEnumImapDataStoreOptions
{
	/*! none*/
	EmaiDataStoreAttr,

	/*! data*/
	EmaiDataStoreString
};

/*!
// @brief	This structure identifies the message data to be stored
//			using the EmaiImapStore or Emai::ImapSession::Store functions.
//
// @see		Emai::ImapSession::Store
// @see		EmaiImapStore
*/
typedef struct
{
	/*! version of the structure*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;		

	/*! EmaiDataStoreAttr or EmaiDataStoreString*/
	EmaiUint32					selector;
	union
	{
		struct
		{
			/*! EmaiStoreSetFlags, etc*/
			EmaiUint32					options;

			/*! message flags (EmaiAnsweredFlag, etc)*/
			EmaiFlags					flags;
		}						attr;

		/*! data to be stored for message*/
		EmaiConstUniCharPtr		data;
	} param;
} EmaiStoreMessageData;

/*!
// @brief	The STORE command alters data associated with a message in the
//			mailbox.\n
//
//			Refer to RFC3501 (chapter 6.4.6) for more information.
//
// @param	inSession - session object
// @param	inMessageSequence - sequence of the message numbers or message UIDs
//						to be stored
// @param	inMessageData - message data to be set
// @param	inOptions - if EmaiImapUIDOption is specified inMessageSequence
//						should specify message UIDs sequense instead of
//						message numbers sequense
//
// @see		Emai::ImapSession::Store
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapStore(
		EmaiImapSession inSession,
		EmaiImapSequenceSet inMessageSequence,
		const EmaiStoreMessageData* inMessageData,
		EmaiOptions inOptions);

/*!
// @brief	The COPY command copies the specified message(s) to the end of the
//			specified destination mailbox. The flags and internal date of the
//			message(s) SHOULD be preserved, and the Recent flag SHOULD be set,
//			in the copy.\n
//
//			If the destination mailbox does not exist, a server SHOULD return
//			an error. It SHOULD NOT automatically create the mailbox.\n
//
//			If the COPY command is unsuccessful for any reason, server
//			implementations MUST restore the destination mailbox to its state
//			before the COPY attempt.\n
//
//			Refer to RFC3501 (chapter 6.4.7) for more information.
//
// @param	inSession - session object
// @param	inMessageSequence - sequence of the message numbers or message UIDs
//						to be copied
// @param	inMailbox - destination mailbox
// @param	inOptions - if EmaiImapUIDOption is specified inMessageSequence
//						should specify message UIDs sequense instead of
//						message numbers sequense
//
// @see		Emai::ImapSession::Copy
//
// @return	Return value
*/
EMAIAPI(EmaiError)
EmaiImapCopy(
		EmaiImapSession inSession,
		EmaiImapSequenceSet inMessageSequence,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions);

#ifdef __cplusplus
	}
#endif

#endif /*__EMAIIMAP_H__*/
