/*!
// @file		EmaiIMAP.hpp
//
// @brief		C++ wrappers for IMAP functionality
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008-2010 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIIMAP_HPP__
#define __EMAIIMAP_HPP__

#include <EmaiIMAP.h>
#include <EmaiMessage.hpp>
#include <EmaiExceptions.hpp>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/
namespace Emai
{

/*!
// @class	ImapSequenceSet
// @brief	Wrapper class for EmaiImapSequenceSet.\n
//			This class allows to easily operate with message sequence sets.
//			It can contains either message numbers on IMAP server or message
//			UIDs.
//
// @see		EmaiImapSequenceSet
*/
class ImapSequenceSet : public Object<EmaiImapSequenceSet>
{
public:
										ImapSequenceSet(
												EmaiOptions inOptions = EmaiNullOptions);

										ImapSequenceSet(
												const ImapSequenceSet& inCopy);
										ImapSequenceSet(
												EmaiImapSequenceSet inSequenceSet,
												bool inAddRef);

										~ImapSequenceSet();

	void								AddNumber(
												EmaiUint32 inNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	void								AddRange(
												EmaiUint32 inMinNumber,
												EmaiUint32 inMaxNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	void								AddSequenceSet(
												const ImapSequenceSet& inSequenceSetToAdd,
												EmaiOptions inOptions = EmaiNullOptions);

	void								RemoveNumber(
												EmaiUint32 inNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	void								RemoveRange(
												EmaiUint32 inMinNumber,
												EmaiUint32 inMaxNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	void								RemoveSequenceSet(
												const ImapSequenceSet& inSequenceSetToRemove,
												EmaiOptions inOptions = EmaiNullOptions);

	EmaiUint32							GetCount(
												EmaiOptions inOptions = EmaiNullOptions) const;
	void								GetBounds(
												EmaiUint32& outFirstNumber,
												EmaiUint32& outLastNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	EmaiUint32							GetFirstNumber(
												EmaiOptions inOptions = EmaiNullOptions);
	EmaiUint32							GetLastNumber(
												EmaiOptions inOptions = EmaiNullOptions);

	bool								GetNextNumberForward(
												EmaiUint32 inNumber,
												EmaiUint32& outNextNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	bool								GetNextNumberBackward(
												EmaiUint32 inNumber,
												EmaiUint32& outNextNumber,
												EmaiOptions inOptions = EmaiNullOptions);
	bool								ContainsNumber(
												EmaiUint32 inNumber,
												EmaiOptions inOptions = EmaiNullOptions);
};


/*!
// @class	ImapStatusCallback
// @brief	Wrapper class for EmaiImapStatusCallback.\n
//			This class contains the (virtual) functionality, which is 
//			supposed to process the progress update reports.
//
// @see		EmaiImapStatusCallback
*/
class ImapStatusCallback
{
friend class ImapSession;

	/*!
	// @brief	Virtual function
	//
	// @param	Emai::ImapSession inSession - session object of the event
	// @param	EmaiUint32 inOperationCode - operation code
	// @param	EmaiConstCharPtr inStatusText - status text
	*/
	virtual EmaiError					OnOperation(
												Emai::ImapSession& inSession,
												EmaiUint32 inOperationCode) = 0;

	/*!
	// @brief	Virtual function; will be triggered periodically while 
	//				sending the data, after a certain amount of data is sent.
	//
	// @param	Emai::ImapSession inSession - session object of the event
	// @param	EmaiUint32 inSentSize - data sent amount, sinse last function call
	// @param	EmaiUint32 inTotalSize - total data amout to be sent
	*/
	virtual EmaiError					OnDataSent(
												Emai::ImapSession& inSession,
												EmaiUint32 inSentSize,
												EmaiUint32 inTotalSize) = 0;

	/*!
	// @brief	Virtual function; will be triggered periodically while 
	//				receiving the data, after a certain amount of data is received.
	//
	// @param	Emai::ImapSession inSession - session object of the event
	// @param	EmaiUint32 inSentSize - data sent amount, sinse last function call
	// @param	EmaiUint32 inTotalSize - total data amout to be sent
	*/
	virtual EmaiError					OnDataReceived(
												Emai::ImapSession& inSession,
												EmaiUint32 inReceivedSize,
												EmaiUint32 inTotalSize) = 0;

	/*!
	// @brief	Virtual function; Will be triggered in case if server returns
	//				an error.
	//
	// @param	Emai::ImapSession inSession - session object of the event
	// @param	EmaiConstCharPtr inServerReply - error reply string
	// @param	EmaiUint32 inReplyCode - error reply code
	*/
	virtual EmaiError					OnServerError(
												Emai::ImapSession& inSession,
												EmaiConstCharPtr inServerReply,
												EmaiUint32 inReplyCode) = 0;
public:
	virtual								~ImapStatusCallback();

protected:
	static EmaiError					StatusCallback(
												EmaiImapSession inSession,
												const EmaiImapCallbackData* inData,
												void* inUserData);
};

/*!
// @class	ImapResponseHandler
// @brief	This class contains the (virtual) functionality, which is 
//			supposed to process the responses returned by the server.
//
// @see		EmaiEnumImapStatusResponseFlags
// @see		EmaiEnumImapStatusResponseTypes
//
// @see		EmaiStatusResponse
// @see		EmaiCapabilityResponse
// @see		EmaiMailboxStatusResponse
*/
class ImapResponseHandler
{
friend class ImapSession;
protected:

	virtual void						ProcessResponse(
												const EmaiImapCommandData* inCommandData,
												const EmaiResponse& inResponse);

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a STATUS command response.
	//
	// @param	EmaiStatusResponse& inResponse - server response
	*/
	virtual void						ProcessStatus(
												const EmaiImapCommandData* inCommandData,
												const EmaiStatusResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a CAPABILITY command response.
	//
	// @param	EmaiCapabilityResponse& inResponse - server response
	*/
	virtual void						ProcessCapability(
												const EmaiCapabilityResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a LIST command response.
	//
	// @param	EmaiListResponse& inResponse - server response
	*/
	virtual void						ProcessList(
												const EmaiListResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a LSUB command response.
	//
	// @param	EmaiLsubResponse& inResponse - server response
	*/
	virtual void						ProcessLsub(
												const EmaiLsubResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a mailbox STATUS command response.
	//
	// @param	EmaiMailboxStatusResponse& inResponse - server response
	*/
	virtual void						ProcessMailboxStatus(
												const EmaiMailboxStatusResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a SEARCH command response.
	//
	// @param	EmaiSearchResponse& inResponse - server response
	*/
	virtual void						ProcessSearch(
												EmaiConstUniCharPtr inMailboxName,
												const EmaiSearchResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a SELECT or EXAMINE commands response.
	//
	// @param	EmaiFlagsResponse& inResponse - server response
	*/
	virtual void						ProcessFlags(
												EmaiConstUniCharPtr inMailboxName,
												const EmaiFlagsResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a EXISTS command response.
	//
	// @param	EmaiSearchResponse& inResponse - server response
	*/
	virtual void						ProcessExists(
												EmaiConstUniCharPtr inMailboxName,
												const EmaiExistsResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a RECENT command response.
	//
	// @param	EmaiRecentResponse& inResponse - server response
	*/
	virtual void						ProcessRecent(
												EmaiConstUniCharPtr inMailboxName,
												const EmaiRecentResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a EXPUNGE command response.
	//
	// @param	EmaiExpungeResponse& inResponse - server response
	*/
	virtual void						ProcessExpunge(
												EmaiConstUniCharPtr inMailboxName,
												const EmaiExpungeResponse& inResponse) = 0;

	/*!
	// @brief	Virtual function; Will be triggered when server returns
	//				a FETCH command response.
	//
	// @param	EmaiFetchResponse& inResponse - server response
	*/
	virtual void						ProcessFetch(
												EmaiConstUniCharPtr inMailboxName,
												const EmaiFetchResponse& inResponse) = 0;

	static void							ProcessResponses(
												EmaiImapSession inSession,
												const EmaiImapCommandData* inCommandData,
												const EmaiResponse* inResponse,
												void* inUserData);
public:
	virtual								~ImapResponseHandler();
	
};

/*!
// @class	ImapSession
// @brief	Wrapper class for EmaiImapSession
//
// @see		Emai::EmaiImapSession
*/
class ImapSession : public Object<EmaiImapSession>
{
protected:

										ImapSession(); //no implementation

public:
										
										ImapSession(
												ImapResponseHandler* inResponseHandler,
												ImapStatusCallback& inStatusCallback,
												EmaiUint32 inTimeout,
												EmaiOptions inOptions = EmaiNullOptions);
										ImapSession(
												EmaiImapResponseCallback inResponseCallback,
												void* inResponseUserData,
												EmaiImapStatusCallback inStatusCallback,
												void* inStatusUserData,
												EmaiUint32 inTimeout,
												EmaiOptions inOptions = EmaiNullOptions);

										ImapSession(
												const ImapSession& inCopy);
										ImapSession(
												EmaiImapSession inSession,
												bool inAddRef);

										~ImapSession();

	/*!
	// @brief	Creates IMAP session object, and connects the object
	//			to the specified mail server by calling EmaiImapSessionCreate
	//			and EmaiImapConnect functions accordingly.
	//
	// @param	EmaiConstCharPtr inMailServer	- pointer to null-teminated string which
	//											specifies name or IP address of IMAP server
	// @param	short inPort					- port number of IMAP server
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapSessionCreate
	// @see		EmaiImapConnect
	*/
	void								Connect(
												EmaiConstUniCharPtr inMailServer,
												short inPort = EmaiDefaultImapPort,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Creates IMAP session object, and connects the object
	//			to the specified mail server by calling EmaiImapSessionCreate
	//			and EmaiImapConnect functions accordingly.
	//
	// @param	EmaiConstCharPtr inMailServer	- pointer to null-teminated string which
	//											specifies name or IP address of IMAP server
	// @param	short inPort					- port number of IMAP server
	// @param	inProxyInfo - proxy server information struct
	// @param	inUserData - user data specified parameter, use NULL
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapSessionCreate
	// @see		EmaiImapConnectEx
	*/
	void								ConnectEx(
												EmaiConstUniCharPtr inMailServer,
												short inPort = EmaiDefaultImapPort,
												const EmaiProxyInfo* inProxyInfo = NULL,
												void* inUserData = NULL,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Disconnects IMAP session object from mail server
	//			by calling the EmaiImapDisconnect function.
	//
	// @param	EmaiOptions inOptions	- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapDisconnect
	*/
	
	void								Disconnect(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Authenticates IMAP session, by calling the EmaiImapAuthenticate
	//			function for the session.
	//
	// @param	EmaiConstCharPtr inAccount	- pointer to null-teminated string to specify
	//										account name for authentication
	// @param	EmaiConstCharPtr inPassword	- pointer to null-teminated string to specify
	//										password for authentication
	// @param	EmaiOptions inOptions		- Authentication method to be used. For example:
	//										EmaiIMAPAuthAutomatic, EmaiIMAPAuthLogin, etc
	//
	// @see		EmaiImapAuthenticate
	// @see		EmaiEnumImapAuthModes
	*/
	void								Authenticate(
												EmaiConstUniCharPtr inAccount,
												EmaiConstUniCharPtr inPassword,
												EmaiOptions inOptions = EmaiIMAPAuthAutomatic);

	/*!
	// @brief	The SELECT command selects a mailbox so that messages in the
	//			mailbox can be accessed.
	//			Only one mailbox can be selected at a time in a connection;
	//			simultaneous access to multiple mailboxes requires multiple
	//			connections. The SELECT command automatically deselects any
	//			currently selected mailbox before attempting the new selection.
	//			Consequently, if a mailbox is selected and a SELECT command that
	//			fails is attempted, no mailbox is selected.\n
	//
	//			Refer to RFC3501 (chapter 6.3.1) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapSelectMailbox
	*/
	void								SelectMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The EXAMINE command is identical to SELECT and returns the same
	//			output; however, the selected mailbox is identified as read-only.
	//			No changes to the permanent state of the mailbox, including
	//			per-user state, are permitted; in particular, EXAMINE MUST NOT
	//			cause messages to lose the \\Recent flag.\n
	//
	//			Refer to RFC3501 (chapter 6.3.2) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapExamineMailbox
	*/
	void								ExamineMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The CREATE command creates a mailbox with the given name. A positive
	//			response is returned only if a new mailbox with that name has been
	//			created. It is an error to attempt to create INBOX or a mailbox
	//			with a name that refers to an extant mailbox. Any error in
	//			creation will return an apropriate error code.\n
	//
	//			Refer to RFC3501 (chapter 6.3.3) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapCreateMailbox
	*/
	void								CreateMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The DELETE command permanently removes the mailbox with the given
	//			name. A positive response is returned only if the mailbox has
	//			been deleted. It is an error to attempt to delete INBOX or a
	//			mailbox name that does not exist.\n
	//
	//			Refer to RFC3501 (chapter 6.3.4) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapDeleteMailbox
	*/
	void								DeleteMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The RENAME command changes the name of a mailbox. A positive
	//			response is returned only if the mailbox has been renamed. It is
	//			an error to attempt to rename from a mailbox name that does not
	//			exist or to a mailbox name that already exists. Any error in
	//			renaming will return an apropriate error code.\n
	//
	//			Renaming INBOX is permitted, and has special behavior. It moves
	//			all messages in INBOX to a new mailbox with the given name,
	//			leaving INBOX empty. If the server implementation supports
	//			inferior hierarchical names of INBOX, these are unaffected by a
	//			rename of INBOX.\n
	//
	//			Refer to RFC3501 (chapter 6.3.5) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox			- original mailbox to be renamed
	// @param	EmaiConstUniCharPtr inNewMailboxName	- new name for the mailbox
	// @param	EmaiOptions inOptions					- not used in current version,
	//														use EmaiNullOptions
	//
	// @see		EmaiImapRenameMailbox
	*/
	void								RenameMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiConstUniCharPtr inNewMailboxName,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The SUBSCRIBE command adds the specified mailbox name to the
	//			server's set of "active" or "subscribed" mailboxes as returned by
	//			the LSUB command. This command returns a positive response only
	//			if the subscription is successful.\n
	//
	//			Refer to RFC3501 (chapter 6.3.6) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- not used in current version,
	//												use EmaiNullOptions
	//
	// @see		EmaiImapSubscribeMailbox
	*/
	void								SubscribeMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The UNSUBSCRIBE command removes the specified mailbox name from
	//			the server's set of "active" or "subscribed" mailboxes as returned
	//			by the LSUB command. This command returns a positive response
	//			only if the unsubscription is successful.\n
	//
	//			Refer to RFC3501 (chapter 6.3.7) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- not used in current version,
	//												use EmaiNullOptions
	//
	// @see		EmaiImapUnsubscribeMailbox
	*/
	void								UnsubscribeMailbox(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The LIST command returns a subset of names from the complete set
	//			of all names available to the client. Zero or more untagged LIST
	//			replies are returned, containing the name attributes, hierarchy
	//			delimiter, and name; see the description of the LIST reply for
	//			more detail.\n
	//
	//			The character "*" is a wildcard, and matches zero or more
	//			characters at this position. The character "%" is similar to "*",
	//			but it does not match a hierarchy delimiter.\n
	//
	//			Refer to RFC3501 (chapter 6.3.8) for more information.
	//
	// @param	EmaiConstUniCharPtr inReferenceName			- reference name
	// @param	EmaiConstUniCharPtr inWildcardedMailboxName	- mailbox name with
	//															possible wildcards
	// @param	EmaiOptions inOptions						- not used in current version,
	//															use EmaiNullOptions
	//
	// @see		EmaiImapListMailboxes
	*/
	void								ListMailboxes(
												EmaiConstUniCharPtr inReferenceName,
												EmaiConstUniCharPtr inWildcardedMailboxName,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The LSUB command returns a subset of names from the set of names
	//			that the user has declared as being "active" or "subscribed".
	//			Zero or more untagged LSUB replies are returned. The arguments to
	//			LSUB are in the same form as those for LIST.\n
	//
	//			Refer to RFC3501 (chapter 6.3.9) for more information.
	//
	// @param	EmaiConstUniCharPtr inReferenceName			- reference name
	// @param	EmaiConstUniCharPtr inWildcardedMailboxName	- mailbox name with
	//															possible wildcards
	// @param	EmaiOptions inOptions						- not used in current version,
	//															use EmaiNullOptions
	//
	// @see		EmaiImapListSubscribedMailboxes
	*/
	void								ListSubscribedMailboxes(
												EmaiConstUniCharPtr inReferenceName,
												EmaiConstUniCharPtr inWildcardedMailboxName,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The STATUS command requests the status of the indicated mailbox.
	//			It does not change the currently selected mailbox, nor does it
	//			affect the state of any messages in the queried mailbox (in
	//			particular, STATUS MUST NOT cause messages to lose the \\Recent
	//			flag).\n
	//
	//			The STATUS command provides an alternative to opening a second
	//			IMAP4rev1 connection and doing an EXAMINE command on a mailbox to
	//			query that mailbox's status without deselecting the current
	//			mailbox in the first IMAP4rev1 connection.\n
	//
	//			Refer to RFC3501 (chapter 6.3.10) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiOptions inOptions			- EmaiMessagesNumber, etc
	//
	// @see		EmaiImapMailboxStatus
	// @see		EmaiEnumImapMailboxStatus
	*/
	void								MailboxStatus(
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiAllStatusData);

	/*!
	// @brief	The APPEND command appends the new message
	//			to the end of the specified destination mailbox. This argument
	//			SHOULD be in the format of an [RFC-2822] message. 8-bit
	//			characters are permitted in the message.\n
	//
	//			If the destination mailbox does not exist, a server MUST return an
	//			error, and MUST NOT automatically create the mailbox.
	//			If the mailbox is currently selected, the normal new message
	//			actions SHOULD occur.\n
	//
	//			Refer to RFC3501 (chapter 6.3.11) for more information.
	//
	// @param	EmaiConstUniCharPtr inMailbox	- mailbox name
	// @param	EmaiConstCharPtr inMessageFlags	- zero terminated string with space delimited
	//											message flags (for example: "\\Draft \\Flagged")
	// @param	EmaiConstCharPtr inDateTime		- zero terminated date/time string, which indicates
	//											internal message date
	//											(for example: " 1-Jan-2006 21:34:27 +0200")
	// @param	const Message& inMessage		- message object to be appended
	// @param	EmaiOptions inOptions			- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapAppendMessage
	*/
	void								AppendMessage(
												EmaiConstUniCharPtr inMailbox,
												const EmaiFlags* inMessageFlags,
												EmaiConstUniCharPtr inDateTime,
												const Message& inMessage,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The CHECK command requests a checkpoint of the currently selected
	//			mailbox. A checkpoint refers to any implementation-dependent
	//			housekeeping associated with the mailbox (e.g., resolving the
	//			server's in-memory state of the mailbox with the state on its
	//			disk) that is not normally executed as part of each command. A
	//			checkpoint MAY take a non-instantaneous amount of real time to
	//			complete. If a server implementation has no such housekeeping
	//			considerations, CHECK is equivalent to NOOP.\n
	//
	//			Refer to RFC3501 (chapter 6.4.1) for more information.
	//
	// @param	EmaiOptions inOptions	- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapCheck
	*/
	void								Check(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Permanently removes all messages that have \Deleted flag set from
	//			the currently selected mailbox, and returns to the authenticated
	//			state from the selected state.\n
	//
	//			Refer to RFC3501 (chapter 6.4.2) for more information.
	//
	// @param	EmaiOptions inOptions	- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapCloseMailbox
	*/
	void								CloseMailbox(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Permanently removes all messages that have \Deleted flag set from
	//			the currently selected mailbox. \n
	//
	//			Refer to RFC3501 (chapter 6.4.3) for more information.
	//
	// @param	const ImapSequenceSet& inMessageUIDSequence - sequence of the
										message UIDs to be expunged
	// @param	EmaiOptions inOptions	- EmaiImapNoCustomExpungeOption or EmaiNullOptions
	//
	// @see		EmaiImapExpunge
	// @see		EmaiEnumImapExpungeOptions
	*/
	void								Expunge(
												const ImapSequenceSet& inMessageUIDSequence,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The SEARCH command searches the mailbox for messages that match
	//			the given searching criteria. Searching criteria consist of one
	//			or more search keys.\n
	//
	//			When multiple keys are specified, the result is the intersection
	//			(AND function) of all the messages that match those keys. For
	//			example, the criteria DELETED FROM "SMITH" SINCE 1-Feb-1994 refers
	//			to all deleted messages from Smith that were placed in the mailbox
	//			since February 1, 1994. A search key can also be a parenthesized
	//			list of one or more search keys (e.g., for use with the OR and NOT
	//			keys).\n
	//
	//			Refer to RFC3501 (chapter 6.4.4) for more information.
	//
	// @param	const EmaiSearchingCriteria* inSearchingCriteria
	//									- if NULL all messages in mailbox will be found
	//																	
	// @param	EmaiOptions inOptions	- specify EmaiImapUIDOption if you want to receive message UIDs
	//										instead of message numbers in SEARCH response
	//
	// @see		EmaiImapSearch
	// @see		EmaiEnumImapUIDOptions
	// @see		EmaiEnumImapSearchOptions
	// @see		EmaiSearchingKey
	// @see		EmaiEnumImapSearching
	// @see		EmaiSearchingCriteria
	*/
	void								Search(
												const EmaiSearchingCriteria* inSearchingCriteria,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The FETCH command retrieves data associated with a message in the
	//			mailbox. The data items to be fetched can be either a single atom
	//			or a parenthesized list.\n
	//
	//			Refer to RFC3501 (chapter 6.4.5) for more information.
	//
	// @param	const ImapSequenceSet& inMessageSequence		- sequence of the messages to be fetched
	// @param	const EmaiFetchMessageData* inMessageData	- message data to be retrieved
	// @param	EmaiOptions inOptions						- if EmaiImapUIDOption is specified inMsgSequence
	//														should specify message UIDs sequense instead of
	//														message numbers sequense
	//
	// @see		EmaiImapFetch
	// @see		EmaiEnumImapFetchOptions
	// @see		EmaiFetchMessageAttribute
	// @see		EmaiEnumImapDataFetchOptions
	// @see		EmaiFetchMessageData
	*/
	void								Fetch(
												const ImapSequenceSet& inMessageSequence,
												const EmaiFetchMessageData* inMessageData,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The STORE command alters data associated with a message in the
	//			mailbox.\n
	//
	//			Refer to RFC3501 (chapter 6.4.6) for more information.
	//
	// @param	const ImapSequenceSet& inMessageSequence	- sequence of the messages
	// @param	const EmaiStoreMessageData* inMessageData	- message data to be set
	// @param	EmaiOptions inOptions						- if EmaiImapUIDOption is specified inMsgSequence
	//														should specify message UIDs sequense instead of
	//														message numbers sequense
	//
	// @see		EmaiImapStore
	// @see		EmaiEnumImapStoreOptions
	// @see		EmaiEnumImapDataStoreOptions
	*/
	void								Store(
												const ImapSequenceSet& inMessageSequence,
												const EmaiStoreMessageData* inMessageData,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	The COPY command copies the specified message(s) to the end of the
	//			specified destination mailbox. The flags and internal date of the
	//			message(s) SHOULD be preserved, and the Recent flag SHOULD be set,
	//			in the copy.\n
	//
	//			If the destination mailbox does not exist, a server SHOULD return
	//			an error. It SHOULD NOT automatically create the mailbox.\n
	//
	//			If the COPY command is unsuccessful for any reason, server
	//			implementations MUST restore the destination mailbox to its state
	//			before the COPY attempt.\n
	//
	//			Refer to RFC3501 (chapter 6.4.7) for more information.
	//
	// @param	const ImapSequenceSet& inMessageSequence	- sequence of the messages to be copied
	// @param	EmaiConstUniCharPtr inMailbox				- destination mailbox
	// @param	EmaiOptions inOptions						- if EmaiImapUIDOption is specified inMsgSequence
	//														should specify message UIDs sequense instead of
	//														message numbers sequense
	//
	// @see		EmaiImapCopy
	*/
	void								Copy(
												const ImapSequenceSet& inMessageSequence,
												EmaiConstUniCharPtr inMailbox,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Marks IMAP session to be cancelled ASAP
	//
	// @param	EmaiOptions inOptions	- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapCancel()
	*/
	void								Cancel(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Can be used as a periodic poll for new messages or message status updates
	//			during a period of inactivity. \n
	//
	//			Refer to RFC3501 (chapter 6.4.1) for more information
	//
	// @param	EmaiOptions inOptions	- not used in current version, use EmaiNullOptions
	//
	// @see		EmaiImapNoop()
	*/	void								Noop(
												EmaiOptions inOptions = EmaiNullOptions);
};


/*
// ImapSequenceSet class implemetation
*/
inline
ImapSequenceSet::ImapSequenceSet(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetCreate(&mObject, inOptions));
}

inline
ImapSequenceSet::ImapSequenceSet(
		const ImapSequenceSet& inCopy
	):
	Parent(inCopy)
{
}

inline
ImapSequenceSet::ImapSequenceSet(
		EmaiImapSequenceSet inSequenceSet,
		bool inAddRef
	):
	Parent(inSequenceSet, inAddRef)
{
}

inline
ImapSequenceSet::~ImapSequenceSet()
{
}

inline void
ImapSequenceSet::AddNumber(
		EmaiUint32 inNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetAddNumber(mObject, inNumber, inOptions));
}

inline void
ImapSequenceSet::AddRange(
		EmaiUint32 inMinNumber,
		EmaiUint32 inMaxNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetAddRange(mObject, inMinNumber,
							inMaxNumber, inOptions));
}

inline void
ImapSequenceSet::AddSequenceSet(
		const ImapSequenceSet& inSequenceSetToAdd,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetAddSequenceSet(mObject,
							inSequenceSetToAdd, inOptions));
}

inline void
ImapSequenceSet::RemoveNumber(
		EmaiUint32 inNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetRemoveNumber(mObject, inNumber, inOptions));
}

inline void
ImapSequenceSet::RemoveRange(
		EmaiUint32 inMinNumber,
		EmaiUint32 inMaxNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetRemoveRange(mObject, inMinNumber,
							inMaxNumber, inOptions));
}

inline void
ImapSequenceSet::RemoveSequenceSet(
		const ImapSequenceSet& inSequenceSetToRemove,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetRemoveSequenceSet(mObject,
							inSequenceSetToRemove, inOptions));
}

inline EmaiUint32
ImapSequenceSet::GetCount(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	) const
{
	EmaiUint32 count = 0;
	CheckErrorCode(EmaiImapSequenceSetGetCount(mObject, &count, inOptions));
	return count;
}

inline void
ImapSequenceSet::GetBounds(
		EmaiUint32& outFirstNumber,
		EmaiUint32& outLastNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSequenceSetGetBounds(mObject, &outFirstNumber,
							&outLastNumber, inOptions));
}

inline EmaiUint32
ImapSequenceSet::GetFirstNumber(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiUint32 firstNumber = 0;
	CheckErrorCode(EmaiImapSequenceSetGetBounds(mObject, &firstNumber,
							NULL, inOptions));
	return firstNumber;
}

inline EmaiUint32
ImapSequenceSet::GetLastNumber(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiUint32 lastNumber = 0;
	CheckErrorCode(EmaiImapSequenceSetGetBounds(mObject, NULL,
							&lastNumber, inOptions));
	return lastNumber;
}

inline bool
ImapSequenceSet::GetNextNumberForward(
		EmaiUint32 inNumber,
		EmaiUint32& outNextNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	if (inOptions != EmaiNullOptions)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return (EmaiImapSequenceSetGetNextNumber(mObject, inNumber,
							&outNextNumber, EmaiImapForward) == EmaiErrNoErr);
}

inline bool
ImapSequenceSet::GetNextNumberBackward(
		EmaiUint32 inNumber,
		EmaiUint32& outNextNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	if (inOptions != EmaiNullOptions)
		CheckErrorCode(EmaiErrIncorrectParameter);

	return (EmaiImapSequenceSetGetNextNumber(mObject, inNumber,
							&outNextNumber, EmaiImapBackward) == EmaiErrNoErr);
}

inline bool
ImapSequenceSet::ContainsNumber(
		EmaiUint32 inNumber,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	return (EmaiImapSequenceContainsNumber(mObject, inNumber, inOptions) == EmaiErrNoErr);
}


/*
// ImapStatusCallback class implemetation
*/
inline EmaiError
ImapStatusCallback::StatusCallback(
		EmaiImapSession inSession,
		const EmaiImapCallbackData* inData,
		void* inUserData
	)
{
	ImapStatusCallback* callback = reinterpret_cast<ImapStatusCallback*>(inUserData);
	EmaiError error = EmaiErrNoErr;
	try
	{
		if (callback != NULL)
		{
			Emai::ImapSession session(inSession, true);
			switch (inData->selector)
			{
				case EmaiImapSelectorNormal:
					error = callback->OnOperation(session, inData->data.normal.operationCode);
					break;
				case EmaiImapSelectorSending:
					error = callback->OnDataSent(session, inData->data.sending.sentSize,
										inData->data.sending.totalSize);
					break;
				case EmaiImapSelectorReceiving:
					error = callback->OnDataReceived(session, inData->data.receiving.receivedSize,
										inData->data.receiving.totalSize);
					break;
				case EmaiImapSelectorServerError:
					error = callback->OnServerError(session, inData->data.serverError.serverReply,
										inData->data.serverError.replyCode);
					break;
				default:
					error = EmaiErrUnknown;
					break;
			}
		}
	}
	catch (...)
	{
		//do not pass exceptions in library
		error = EmaiErrUnknown;
	}

	return error;
}

inline
ImapStatusCallback::~ImapStatusCallback()
{
}

/*
// EmaiImapSession class implemetation
*/
inline
ImapSession::ImapSession(
		ImapResponseHandler* inResponseHandler,
		ImapStatusCallback& inStatusCallback,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSessionCreate(&mObject,
					&ImapResponseHandler::ProcessResponses, inResponseHandler,
					&ImapStatusCallback::StatusCallback, &inStatusCallback,
					inTimeout, inOptions));
}

inline
ImapSession::ImapSession(
		EmaiImapResponseCallback inResponseCallback,
		void* inResponseUserData,
		EmaiImapStatusCallback inStatusCallback,
		void* inStatusUserData,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSessionCreate(&mObject,
					inResponseCallback, inResponseUserData,
					inStatusCallback, inStatusUserData,
					inTimeout, inOptions));
}

inline
ImapSession::ImapSession(
		const ImapSession& inCopy
	):
	Parent(inCopy)
{
}

inline
ImapSession::ImapSession(
		EmaiImapSession inSession,
		bool inAddRef
	):
	Parent(inSession, inAddRef)
{
}

inline
ImapSession::~ImapSession()
{
}

inline void
ImapSession::Connect(
		EmaiConstUniCharPtr inMailServer,
		short inPort /*= EmaiDefaultImapPort*/,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapConnect(mObject, inMailServer, inPort, inOptions));
}

inline void
ImapSession::ConnectEx(
		EmaiConstUniCharPtr inMailServer,
		short inPort /*= EmaiDefaultImapPort*/,
		const EmaiProxyInfo* inProxyInfo /*= NULL*/,
		void* inUserData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapConnectEx(mObject, inMailServer, inPort, inProxyInfo, inUserData,inOptions));
}

inline void
ImapSession::Disconnect(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapDisconnect(mObject, inOptions));
}

inline void
ImapSession::Authenticate(
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inPassword,
		EmaiOptions inOptions /*= EmaiIMAPAuthAutomatic*/
	)
{
	CheckErrorCode(EmaiImapAuthenticate(mObject, inAccount,
							inPassword, inOptions));
}

inline void
ImapSession::SelectMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSelectMailbox(mObject, inMailbox, inOptions));
}

inline void
ImapSession::ExamineMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapExamineMailbox(mObject, inMailbox, inOptions));
}

inline void
ImapSession::CreateMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapCreateMailbox(mObject, inMailbox, inOptions));
}

inline void
ImapSession::DeleteMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapDeleteMailbox(mObject, inMailbox, inOptions));
}

inline void
ImapSession::RenameMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiConstUniCharPtr inNewMailboxName,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapRenameMailbox(mObject, inMailbox,
						inNewMailboxName, inOptions));
}

inline void
ImapSession::SubscribeMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSubscribeMailbox(mObject, inMailbox, inOptions));
}

inline void
ImapSession::UnsubscribeMailbox(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapUnsubscribeMailbox(mObject, inMailbox, inOptions));
}

inline void
ImapSession::ListMailboxes(
		EmaiConstUniCharPtr inReferenceName,
		EmaiConstUniCharPtr inWildcardedMailboxName,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapListMailboxes(mObject, inReferenceName,
						inWildcardedMailboxName, inOptions));
}

inline void
ImapSession::ListSubscribedMailboxes(
		EmaiConstUniCharPtr inReferenceName,
		EmaiConstUniCharPtr inWildcardedMailboxName,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapListSubscribedMailboxes(mObject, inReferenceName,
						inWildcardedMailboxName, inOptions));
}

inline void
ImapSession::MailboxStatus(
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiAllStatusData*/
	)
{
	CheckErrorCode(EmaiImapMailboxStatus(mObject, inMailbox, inOptions));
}

inline void
ImapSession::AppendMessage(
		EmaiConstUniCharPtr inMailbox,
		const EmaiFlags* inMessageFlags,
		EmaiConstUniCharPtr inDateTime,
		const Message& inMessage,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapAppendMessage(mObject, inMailbox,
						inMessageFlags, inDateTime, inMessage, inOptions));
}

inline void
ImapSession::Check(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapCheck(mObject, inOptions));
}

inline void
ImapSession::CloseMailbox(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapCloseMailbox(mObject, inOptions));
}

inline void
ImapSession::Expunge(
		const ImapSequenceSet& inMessageUIDSequence,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapExpunge(mObject, inMessageUIDSequence, inOptions));
}

inline void
ImapSession::Search(
		const EmaiSearchingCriteria* inSearchingCriteria,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapSearch(mObject, inSearchingCriteria, inOptions));
}

inline void
ImapSession::Fetch(
		const ImapSequenceSet& inMessageSequence,
		const EmaiFetchMessageData* inMessageData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapFetch(mObject, inMessageSequence,
							inMessageData, inOptions));
}

inline void
ImapSession::Store(
		const ImapSequenceSet& inMessageSequence,
		const EmaiStoreMessageData* inMessageData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapStore(mObject, inMessageSequence,
							inMessageData, inOptions));
}

inline void
ImapSession::Copy(
		const ImapSequenceSet& inMessageSequence,
		EmaiConstUniCharPtr inMailbox,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapCopy(mObject, inMessageSequence, inMailbox, inOptions));
}

inline void
ImapSession::Cancel(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapCancel(mObject, inOptions));
}

inline void
ImapSession::Noop(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiImapNoop(mObject, inOptions));
}

/*
// ImapResponseHandler class implementation
*/
inline
ImapResponseHandler::~ImapResponseHandler()
{
}

inline void
ImapResponseHandler::ProcessResponses(
		EmaiImapSession /*inSession*/,
		const EmaiImapCommandData* inCommandData,
		const EmaiResponse* inResponse,
		void* inUserData
	)
{
	ImapResponseHandler* handler = reinterpret_cast<ImapResponseHandler*>(inUserData);
	try
	{
		if (handler != NULL)
			handler->ProcessResponse(inCommandData, *inResponse);
	}
	catch (...)
	{
		//do not pass exceptions in library
	}
}

inline void
ImapResponseHandler::ProcessResponse(
		const EmaiImapCommandData* inCommandData,
		const EmaiResponse& inResponse
	)
{
	switch (inResponse.responseType)
	{
		case EmaiStatusResponseType:
			ProcessStatus(inCommandData, inResponse.data.status);
			break;
		case EmaiCapabilityResponseType:
			ProcessCapability(inResponse.data.capability);
			break;
		case EmaiListResponseType:
			ProcessList(inResponse.data.list);
			break;
		case EmaiLsubResponseType:
			ProcessLsub(inResponse.data.lsub);
			break;
		case EmaiMailboxStatusResponseType:
			ProcessMailboxStatus(inResponse.data.mailboxStatus);
			break;
		case EmaiSearchResponseType:
			assert(inCommandData->sourceMailboxName != NULL);
			ProcessSearch(inCommandData->sourceMailboxName, inResponse.data.search);
			break;
		case EmaiFlagsResponseType:
			assert(inCommandData->sourceMailboxName != NULL);
			ProcessFlags(inCommandData->sourceMailboxName, inResponse.data.flags);
			break;
		case EmaiExistsResponseType:
			assert(inCommandData->sourceMailboxName != NULL);
			ProcessExists(inCommandData->sourceMailboxName, inResponse.data.exists);
			break;
		case EmaiRecentResponseType:
			assert(inCommandData->sourceMailboxName != NULL);
			ProcessRecent(inCommandData->sourceMailboxName, inResponse.data.recent);
			break;
		case EmaiExpungeResponseType:
			assert(inCommandData->sourceMailboxName != NULL);
			ProcessExpunge(inCommandData->sourceMailboxName, inResponse.data.expunge);
			break;
		case EmaiFetchResponseType:
			assert(inCommandData->sourceMailboxName != NULL);
			ProcessFetch(inCommandData->sourceMailboxName, inResponse.data.fetch);
			break;

		case EmaiContinueRequestResponseType:
			break;
		case EmaiUnrecognizedResponseType:
		default:
			CheckErrorCode(EmaiErrUnknown);
	}
}

}


#endif /*__EMAIIMAP_HPP__*/
