/*!
// @file		EmaiPOP3.h
//
// @brief		POP3 functionality
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008-2010 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIPOP3_H__
#define __EMAIPOP3_H__

#include <EmaiTypes.h>
#include <EmaiErrors.h>
#include <EmaiMessage.h>
#include <EmaiProxy.h>

#ifdef __cplusplus
	extern "C" {
#endif

#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef struct EmaiOpaquePop3Session*			EmaiPop3Session;
typedef struct EmaiOpaquePop3MessageInfoList*	EmaiPop3MessageInfoList;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

/***************************************************
//
// Function for communicating with POP3 server
//
***************************************************/

/*!
// @brief	Constants for selector member of the EmaiPop3CallbackData structure
//
// @see		EmaiPop3CallbackData
// @see		Emai::Pop3StatusCallback
*/
enum EmaiEnumPop3Selector
{
	EmaiPop3SelectorNormal			= 0,/*!< Normal*/
	EmaiPop3SelectorReceiving,			/*!< Receiving*/
	EmaiPop3SelectorServerError			/*!< ServerError*/
};

/*!
// @brief	Constants for operation codes for pop3 sessions
//
// @see		EmaiPop3CallbackData
*/
enum EmaiEnumPop3OperationCode
{
	EmaiPop3ConnectingToServer		= 0,
	EmaiPop3ConnectedToServer,
	EmaiPop3Authenticating,
	EmaiPop3Authenticated,
	EmaiPop3GettingMessageList,
	EmaiPop3GotMessageList,
	EmaiPop3GettingMessageUIDLs,
	EmaiPop3GotMessageUIDLs,
	EmaiPop3RetrievingMessage,
	EmaiPop3MessageRetrieved,
	EmaiPop3DeletingMessage,
	EmaiPop3MessageDeleted,
	EmaiPop3SendingNOOP,
	EmaiPop3NOOPSent,
	EmaiPop3ClosingConnection,
	EmaiPop3ConnectionClosed
};

/*!
// @brief	Constants for reply codes for pop3 sessions
//
// @see		EmaiPop3CallbackData
*/
enum EmaiEnumPop3ReplyCode
{
	EmaiPop3NotSupported			= 0,/*!< Reply codes not supported by server*/
	EmaiPop3NoCode,						/*!< Reply code is not specified by server*/
	EmaiPop3Unknown,					/*!< Reply code is not recognized by library*/
	EmaiPop3LoginDelay,					/*!< The login delay period is not expired yet*/
	EmaiPop3InUse,						/*!< Maildrop is currently in use (probably by*/
										/*!< another POP3 client)*/
	EmaiPop3System,						/*!< Failure due to system error*/
	EmaiPop3Auth						/*!< Authentication failed for some reason*/
										/*!< (incorrect password or unknown user name)*/
};

/*!
// @brief	EmaiPop3CallbackData structure
*/
typedef struct
{
	/*! Version of the structure,*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	EmaiUint32					selector;
	union
	{
		/*! selector == EmaiPop3SelectorNormal*/
		struct
		{
			EmaiUint32			operationCode;/*!< @see EmaiEnumPop3OperationCode*/
		} normal;

		/*! selector == EmaiPop3SelectorReceiving*/
		struct
		{
			EmaiUint32			receivedSize;
			EmaiUint32			totalSize;
		} receiving;

		/*! selector == EmaiPop3SelectorServerError*/
		struct
		{
			EmaiConstCharPtr	serverReply;
			EmaiUint32			replyCode;		/*!< @see EmaiEnumPop3ReplyCode*/
		} serverError;
	} data;
} EmaiPop3CallbackData;

/*
// Status callback declararion for POP3 session
*/
#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef EMAICALLBACKAPI(EmaiError, EmaiPop3StatusCallback)(
		EmaiPop3Session inSession,
		const EmaiPop3CallbackData* inData,
		void* inUserData);
#endif /* DOXYGEN_SHOULD_SKIP_THIS */


/*!
// @brief	Constants, used for session establishing function.
//
// @see		EmaiPop3SessionCreate
// @see		Emai::Pop3Session::Pop3Session()
*/
enum EmaiEnumPop3SSL
{
	
	EmaiPop3UseSSLOption	= 0x01, /*!< If this flag is used, this session */
								/* will connect only to secured SMTP servers.*/
};

/*!
// @brief	Creates POP3 session object.
//
// @param	outSession - on exit contains session object
// @param	inStatusCallback - status callback
// @param	inUserData - user data which is passed in status callback
//			as inUserData parameter
// @param	inTimeout - timeout value in seconds
// @param	inOptions - use EmaiUseSSLOption to create secured
//					SSL session or EmaiNullOptions otherwise
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3SessionCreate(
		EmaiPop3Session* outSession,
		EmaiPop3StatusCallback inStatusCallback,		/*can be NULL*/
		void* inUserData,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions);


/*!
// @brief	Constants for default POP3 ports.
//
// @see		EmaiPop3Connect
// @see		Emai::Pop3Session::Connect
*/
enum EmaiEnumPop3Ports
{
	EmaiDefaultPop3Port		= 110,	/*!< Default POP3 port*/
	EmaiDefaultPop3SSLPort	= 995	/*!< Default SSL POP3 port*/
};

/*!
// @brief	Connects POP3 session object to the specified
//			mail server.
//
// @param	inSession - session object to be connected
// @param	inMailServer - pointer to null-teminated string which
//						specifies name or IP address of POP3 server
// @param	inPort - port number of POP3 server
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiEnumPop3Ports
// @see		Emai::Pop3Session::Connect
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3Connect(
		EmaiPop3Session inSession,
		EmaiConstUniCharPtr inMailServer,
		short inPort,
		EmaiOptions inOptions);

/*!
// @brief	Connects POP3 session object to the specified
//			mail server.
//
// @param	inSession - session object to be connected
// @param	inMailServer - pointer to null-teminated string which
//						specifies name or IP address of POP3 server
// @param	inPort - port number of POP3 server
// @param	inProxyInfo - proxy server information struct
// @param	inUserData - user data specified parameter, use NULL
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiEnumPop3Ports
// @see		Emai::Pop3Session::ConnectEx
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3ConnectEx(
		EmaiPop3Session inSession,
		EmaiConstUniCharPtr inMailServer,
		short inPort,
		const EmaiProxyInfo* inProxyInfo,
		void* inUserData,
		EmaiOptions inOptions);

/*!
// @brief	Disconnects POP3 session object from mail server
//
// @param	inSession - session object to be disconnected
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::Pop3Session::Disconnect
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3Disconnect(
		EmaiPop3Session inSession,
		EmaiOptions inOptions);

/*!
// @brief	Constants, Authentication modes for EmaiPop3Authenticate function.
//
// @see EmaiPop3Authenticate
// @see Emai::Pop3Session::Authenticate
*/
enum EmaiEnumPop3AuthSelector
{
	/*!< Automatically tries all available mechanisms*/
	EmaiPOP3AuthAutomatic,

	/*!< USER/PASS authentification*/
	EmaiPOP3AuthLoginCommand,

	/*!< APOP authentification*/
	EmaiPOP3AuthApop,

	/*!< LOGIN authentification*/
	EmaiPOP3AuthLogin,

	/*!< PLAIN authentification*/
	EmaiPOP3AuthPlain,

	/*!< CRAM-MD5 authentification*/
	EmaiPOP3AuthCramMD5,

	/*!< NTLM authentification*/
	EmaiPOP3AuthNTLM
};

/*!
// @brief	Authenticates the POP3 session.
//
// @param	inSession - session object, to be authenticated
// @param	inAccount - pointer to null-teminated string, which conatins
//						account name for authentication
// @param	inPassword - pointer to null-teminated string, which contains
//						password for authentication
// @param	inOptions - preffered authentication method, i.e.
//						EmaiPOP3AuthAutomatic, EmaiPOP3AuthLogin, etc.
//
// @see		EmaiEnumPop3AuthSelector
// @see		Emai::Pop3Session::Authenticate
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3Authenticate(
		EmaiPop3Session inSession,
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inPassword,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the list of messages, stored on the server. This function
//			internally calls EmaiPop3MessageInfoListEvaluateSubList with
//			EmaiMessagesForDownload option, so after calling
//			EmaiPop3RetrieveMessageList you can skip evaluating of download
//			message list and start message downloading immediately.
//
// @param	inSession - session object, to be used
// @param	inMessageInfoList - message list created using
//				EmaiPop3MessageInfoListCreateFromFile or
//				EmaiPop3MessageInfoListCreateFromData functions; initially this
//				list contains messages which were received in previous
//				pop3 sessions. It is up to user to pass correct list
//				which contains message history for the e-mail account
//				currenly user is working with.
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiPop3MessageInfoListCreateFromFile
// @see		EmaiPop3MessageInfoListCreateFromData
// @see		Emai::Pop3Session::RetrieveMessageList
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3RetrieveMessageList(
		EmaiPop3Session inSession,
		EmaiPop3MessageInfoList inMessageInfoList,
		EmaiOptions inOptions);

/*!
// @brief	Retrieve type for EmaiPop3RetrieveMessage function
//
// @see		Emai::Pop3Session::RetrieveMessage
// @see		EmaiPop3RetrieveMessage
*/
enum EmaiEnumPop3RetrieveAttribute
{
	/*!< Retrieve only message header*/
	EmaiRetrieveRFC822Header = 1
};

/*!
// @brief	Retrieves the message from the server.
//
// @param	inSession - session object, to be used
// @param	outMessage - on exit contains message object
// @param	inMessageNumber - one-based message number on server (See @ref EmaiMessageInfo);
//						messageNumber member of EmaiMessageInfo structure
//						contains the number
// @param	inOptions - EmaiRetrieveRFC822Header for retrieving message header only, EmaiNullOptions retrives full message
//
// @see		Emai::Pop3Session::RetrieveMessage
// @see		EmaiMessageInfo
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3RetrieveMessage(
		EmaiPop3Session inSession,
		EmaiMessage* outMessage,
		EmaiUint32 inMessageNumber,
		EmaiOptions inOptions);

/*!
// @brief	Marks message on server as "deleted".
//
// @param	inSession - session object, to be used
// @param	inMessageNumber - message number on the server
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::Pop3Session::DeleteMessage
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3DeleteMessage(
		EmaiPop3Session inSession,
		EmaiUint32 inMessageNumber,
		EmaiOptions inOptions);

/*!
// @brief	Unmarks all messages as "deleted" on the server.
//
// @param	inSession - session object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::Pop3Session::Reset
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3Reset(
		EmaiPop3Session inSession,
		EmaiOptions inOptions);

/*!
// @brief	Marks POP3 session to be cancelled ASAP
//
// @param	inSession - session object, to be used
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3Cancel(
		EmaiPop3Session inSession,
		EmaiOptions inOptions);

/*!
// @brief	Sens NOOP command to the POP3 server.
//			Used to avoid timout disconnection, during inactivity period.
//
// @param	inSession - session object, to be used
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::Pop3Session::Noop
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3Noop(
		EmaiPop3Session inSession,
		EmaiOptions inOptions);


/***************************************************
//
// Function for managing messages history
//
***************************************************/

/*!
// @brief	Constants for EmaiPop3CreateMessageInfoList functions
//
// @see		EmaiPop3MessageInfoListCreateFromFile
// @see		Emai::MessageInfoList::MessageInfoList
*/
enum EmaiEnumPop3CreateMessageInfoFlags
{
	EmaiCreateFileIfNotExists				= 1
};

/*!
// @brief	Creates message info list from the specified file
//
// @param	outMessageInfoList - on exit contains message info list object
// @param	inFullFilePath - full path to the file where info list is stored,
//							relative file paths are not accepted by the function
// @param	inOptions - use EmaiCreateFileIfNotExists if you want to create
//				file, use EmaiNullOptions otherwise
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListCreateFromFile(
		EmaiPop3MessageInfoList* outMessageInfoList,
		EmaiConstUniCharPtr inFullFilePath,
		EmaiOptions inOptions);

/*!
// @brief	Creates message info list from the input data
//
// @param	outMessageInfoList - on exit contains message info list object
// @param	inMessageListData - pointer to the message data, where info list is stored
// @param	inMessageListDataSize - size of the inMessageListData
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListCreateFromData(
		EmaiPop3MessageInfoList* outMessageInfoList,
		const void* inMessageListData,
		EmaiUint32 inMessageListDataSize,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves message info list data
//
// @param	intMessageInfoList - message info list object to be examined
// @param	outData - on exit will be filled with history data
// @param	outDataSize - on exit will be filled with size of history data
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListGetHistoryData(
		EmaiPop3MessageInfoList inMessageInfoList,
		const void** outData,
		EmaiUint32& outDataSize,
		EmaiOptions inOptions);

/*!
// @brief	Reset message info list
//
// @param	intMessageInfoList - message info list object to be examined
// @param	inAccount - reset account string
// @param	inServer - reset server string
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListResetHistory(
		EmaiPop3MessageInfoList inMessageInfoList,
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inServer,
		EmaiOptions inOptions);

/*!
// @brief	Message info stored at the message info list
//
// @see		EmaiPop3MessageInfoListGetInfo
*/
typedef struct
{
	EmaiConstCharPtr				messageUID;			/*!< zero terminated string*/
														/*!< with UID of the message*/
	EmaiUint32						messageNumber;		/*!< one-based number of the*/
														/*!< message on pop3 server*/
	EmaiUint32						messageSize;		/*!< message size declared*/
														/*!< by server, it can differ*/
														/*!< from real message size*/
														/*!< (it depends on server
														/*!< implementation)*/
	EmaiUint32						messageTimeLag;		/*!< in seconds*/
} EmaiMessageInfo;


/*!
// @brief	Constants for EmaiPop3MessageInfoListEvaluateSubList,
//			EmaiPop3MessageInfoListGetCount and EmaiPop3MessageInfoListGetInfo
//			functions
//
// @see		EmaiPop3MessageInfoListGetCount
// @see		Emai::MessageInfoList::GetCount
// @see		EmaiPop3MessageInfoListGetInfo
// @see		Emai::MessageInfoList::GetInfo
// @see		EmaiPop3MessageInfoListEvaluateSubList
// @see		Emai::MessageInfoList::EvaluateSublist
*/
enum EmaiEnumPop3MessageInfoListGetCountFlags
{
	EmaiMessagesForDownload		= 0,	/*!< Get infomation about messages*/
										/*!< which are not received yet*/
	EmaiMessagesForDeletion,			/*!< Get infomation about messages*/
										/*!< which should be deleted*/
	EmaiMessagesOnServer				/*!< Get infomation about messages*/
										/*!< which are currently on server*/
};

/*!
// @brief	Calculates the sublist of the messages, depending of the inOptions value.
//			For EmaiMessagesForDownload value this function evaluates message list
//			which wasn't downloaded from the server yet.
//			For EmaiMessagesForDeletion value this function evaluates outdated
//			messages which were received more that 'inDays' days ago.
//
// @param	inMessageInfoList - message info list object to be examined
// @param	inDays - number of days
// @param	inOptions - EmaiMessagesForDownload or EmaiMessagesForDeletion or
//						EmaiMessagesOnServer (See @ref EmaiEnumPop3MessageInfoListGetCountFlags)
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListEvaluateSubList(
		EmaiPop3MessageInfoList inMessageInfoList,
		EmaiUint32 inDays,
		EmaiOptions inOptions);

/*!
// @brief	Returns the number of information records, stored at the message info list.
//
// @param	inMessageInfoList - message info list object to be examined
// @param	outCount - on exit contains the number of information records, stored at the message info list
// @param	inOptions - EmaiMessagesForDownload or EmaiMessagesForDeletion or
//						EmaiMessagesOnServer (See @ref EmaiEnumPop3MessageInfoListGetCountFlags)
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListGetCount(
		EmaiPop3MessageInfoList inMessageInfoList,
		EmaiUint32* outCount,
		EmaiOptions inOptions);


/*!
// @brief	Retrives information record stored at the particular
//			index of the message info list.
//
// @param	inMessageInfoList - message info list object, to be examined
// @param	inMessageIndex - zero-based index of the record
// @param	outInfo - pointer to the EmaiMessageInfo structure, which is
//				filled with message information, on successful
//				function exit
// @param	inOptions - EmaiMessagesForDownload or EmaiMessagesForDeletion or
//						EmaiMessagesOnServer (See @ref EmaiEnumPop3MessageInfoListGetCountFlags)
//
// @see		EmaiMessageInfo
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoListGetInfo(
		EmaiPop3MessageInfoList inMessageInfoList,
		EmaiUint32 inMessageIndex,
		EmaiMessageInfo* outInfo,
		EmaiOptions inOptions);


/*!
// @brief	Constants for the EmaiPop3MessageInfoMarkMessageReceived function
//
// @see		EmaiPop3MessageInfoMarkMessageReceived
// @see		Emai::MessageInfoList::MarkMessageReceived
*/
enum EmaiEnumPop3MessageInfoMarkMessageReceivedFlags
{
	EmaiMessageIndexInList		= 0,	/*!< inMessageIndex parameter is zero-based*/
										/*!< index in EmaiMessagesForDownload sublist*/
	EmaiMessageIndexOnServer			/*!< inMessageIndex is one-based message*/
										/*!< number on pop3 server*/
};

/*!
// @brief	Marks the message at the specified index as "received", and saves it's info
//			in the message history. This function will fail, if current sublist is
//			not EmaiMessagesForDownload.
//
// @param	inMessageInfoList - message info list object to be modified
// @param	inMessageIndex - index of the message, actual interpretation of
//					this parameter depends of the inOptions value, for
//					more information see options value description
// @param	inOptions - EmaiMessageIndexInList or EmaiMessageIndexOnServer
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiPop3MessageInfoMarkMessageReceived(
		EmaiPop3MessageInfoList inMessageInfoList,
		EmaiUint32 inMessageIndex,
		EmaiOptions inOptions);


#ifdef __cplusplus
	}
#endif

#endif /*__EMAIPOP3_H__*/
