/*!
// @file		EmaiSMTP.hpp
//
// @brief		C++ wrappers for SMTP functionality
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAISMTP_HPP__
#define __EMAISMTP_HPP__

#include <EmaiSMTP.h>
#include <EmaiMessage.hpp>
#include <EmaiExceptions.hpp>

/*!
// @namespace	Emai
// @brief		Mailit for C/C++ library namespace
// @author		(C) 2006 Dacons LLP.
*/
namespace Emai
{

/*!
// @class	SmtpStatusCallback
// @brief	Wrapper class for EmaiSmtpStatusCallback
//
// @see		Emai::SmtpSession::SmtpSession
*/
class SmtpStatusCallback
{
friend class SmtpSession;

	/*!
	// @brief	Virtual function
	//
	// @param	EmaiSmtpSession inSession - session object of the event
	// @param	EmaiUint32 inOperationCode - operation code
	*/
	virtual EmaiError					OnOperation(
												Emai::SmtpSession& inSession,
												EmaiUint32 inOperationCode) = 0;

	/*!
	// @brief	Virtual function; will be triggered periodically while 
	//				sending the message, after a certain amount of data is sent.
	//
	// @param	EmaiSmtpSession inSession - session object of the event
	// @param	EmaiUint32 inSentSize - data sent amount, sinse last function call
	// @param	EmaiUint32 inTotalSize - total data amout to be sent
	*/
	virtual EmaiError					OnDataSent(
												Emai::SmtpSession& inSession,
												EmaiUint32 inSentSize,
												EmaiUint32 inTotalSize) = 0;

	/*!
	// @brief	Virtual function; Will be triggered in case if server returns
	//				an error.
	//
	// @param	EmaiSmtpSession inSession - session object of the event
	// @param	EmaiConstCharPtr inServerReply - error reply string
	// @param	EmaiUint32 inReplyCode - error reply code
	*/
	virtual EmaiError					OnServerError(
												Emai::SmtpSession& inSession,
												EmaiConstCharPtr inServerReply,
												EmaiUint32 inReplyCode) = 0;

public:
	virtual								~SmtpStatusCallback();

protected:
	static EmaiError					StatusCallback(
												EmaiSmtpSession inSession,
												const EmaiSmtpCallbackData* inData,
												void* inUserData);
};


/*!
// @class  SmtpSession
// @brief  Wrapper class for EmaiSmtpSession
*/
class SmtpSession : public Object<EmaiSmtpSession>
{
protected:

	typedef Object<EmaiSmtpSession>		Parent;

										//! no implementation
										SmtpSession();

public:
										
										SmtpSession(
												SmtpStatusCallback& inStatusCallback,
												EmaiUint32 inTimeout,
												EmaiOptions inOptions = EmaiNullOptions);
										SmtpSession(
												EmaiSmtpStatusCallback inStatusCallback,
												void* inUserData,
												EmaiUint32 inTimeout,
												EmaiOptions inOptions = EmaiNullOptions);

										SmtpSession(
												const SmtpSession& inCopy);
										SmtpSession(
												EmaiSmtpSession inSession,
												bool inAddRef);

										~SmtpSession();

	/*!
	// @brief	Establishes connection to the server.
	//
	// @param	EmaiConstCharPtr inMailServer - pointer to the null-teminated string,
	//							which contains the name or IP address of the SMTP server
	// @param	short inPort - port number of the SMTP server
	// @param	EmaiOptions inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiEnumSmtpDefaultPort
	// @see		EmaiSmtpConnect()
	*/
	void								Connect(
												EmaiConstUniCharPtr inMailServer,
												short inPort = EmaiDefaultSmtpPort,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Establishes connection to the server.
	//
	// @param	EmaiConstCharPtr inMailServer - pointer to the null-teminated string,
	//							which contains the name or IP address of the SMTP server
	// @param	short inPort - port number of the SMTP server
	// @param	inProxyInfo - proxy server information struct
	// @param	inUserData - user data specified parameter, use NULL
	// @param	EmaiOptions inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiEnumSmtpDefaultPort
	// @see		EmaiSmtpConnectEx()
	*/
	void								ConnectEx(
												EmaiConstUniCharPtr inMailServer,
												short inPort = EmaiDefaultSmtpPort,
												const EmaiProxyInfo* inProxyInfo = NULL,
												void* inUserData = NULL,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Disconnects the session from the server.
	// 
	// @param	EmaiOptions inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiSmtpDisconnect()
	*/
	void								Disconnect(
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Authenticates the session at the server.
	//
	// @param	EmaiConstCharPtr inAccount - pointer to the null-teminated string,
	//							which conatins account name for authentication
	// @param	EmaiConstCharPtr inPassword - pointer to null-teminated string,
	//							which contains password for authentication
	// @param	EmaiOptions inOptions - preffered authentication method, i.e.
	//							EmaiSMTPAuthAutomatic, EmaiSMTPAuthLogin etc
	//
	// @see		EmaiEnumSmtpAuthSelector
	// @see		EmaiSmtpAuthenticate()
	*/
	void								Authenticate(
												EmaiConstUniCharPtr inAccount,
												EmaiConstUniCharPtr inPassword,
												EmaiOptions inOptions = EmaiSMTPAuthAutomatic);

	/*!
	// @brief	Sends the message.
	//
	// @param	EmaiMessage inMessage - message to be sent
	// @param	EmaiOptions inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiSmtpSendMessage()
	*/
	void								SendMessage(
												const Message& inMessage,
												EmaiOptions inOptions = EmaiNullOptions);

	/*!
	// @brief	Sends the message.
	//
	// @param	EmaiMessage inMessage - message to be sent
	// @param	outFailedRecipients - failes recipients list
	// @param	EmaiOptions inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiSmtpSendMessage()
	*/
	void								SendMessageEx(
												const Message& inMessage,
												StringList& outFailedRecipients,
												EmaiOptions inOptions = EmaiNullOptions);


	/*!
	// @brief	Marks the SMTP session to be cancelled ASAP.
	//
	// @param	EmaiOptions inOptions
	//
	// @see		EmaiSmtpCancel()
	*/
	void								Cancel(
												EmaiOptions inOptions = EmaiNullOptions);
												
												
	/*!
	// @brief	Sens NOOP command to the SMTP server.
	//			Used to avoid timout disconnection, during inactivity period.
	//
	// @param	inOptions - not used in current version, use EmaiNullOptions
	//
	// @see		EmaiSmtpNoop()
	*/
	void								Noop(
												EmaiOptions inOptions = EmaiNullOptions);
};


/*
// SmtpStatusCallback class implemetation
*/
inline EmaiError
SmtpStatusCallback::StatusCallback(
		EmaiSmtpSession inSession,
		const EmaiSmtpCallbackData* inData,
		void* inUserData
	)
{
	SmtpStatusCallback* callback = reinterpret_cast<SmtpStatusCallback*>(inUserData);
	EmaiError error = EmaiErrNoErr;
	try
	{
		if (callback != NULL)
		{
			Emai::SmtpSession session(inSession, true);
			switch (inData->selector)
			{
				case EmaiSmtpSelectorNormal:
					error = callback->OnOperation(session,
										inData->data.normal.operationCode);
					break;
				case EmaiSmtpSelectorSending:
					error = callback->OnDataSent(session,
										inData->data.sending.sentSize,
										inData->data.sending.totalSize);
					break;
				case EmaiSmtpSelectorServerError:
					error = callback->OnServerError(session,
										inData->data.serverError.serverReply,
										inData->data.serverError.replyCode);
					break;
				default:
					error = EmaiErrUnknown;
					break;
			}
		}
	}
	catch (...)
	{
		//do not pass exceptions in library
		error = EmaiErrUnknown;
	}

	return error;
}

inline
SmtpStatusCallback::~SmtpStatusCallback()
{
}

/*
// SmtpSession class implemetation
*/
inline
SmtpSession::SmtpSession(
		SmtpStatusCallback& inStatusCallback,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpSessionCreate(&mObject,
					&SmtpStatusCallback::StatusCallback, &inStatusCallback,
					inTimeout, inOptions));
}

inline
SmtpSession::SmtpSession(
		EmaiSmtpStatusCallback inStatusCallback,
		void* inUserData,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpSessionCreate(&mObject,
					inStatusCallback, inUserData,
					inTimeout, inOptions));
}

inline
SmtpSession::SmtpSession(
		const SmtpSession& inCopy
	):
	Parent(inCopy)
{
}

inline
SmtpSession::SmtpSession(
		EmaiSmtpSession inSession,
		bool inAddRef
	):
	Parent(inSession, inAddRef)
{
}

inline
SmtpSession::~SmtpSession()
{
}

inline void
SmtpSession::Connect(
		EmaiConstUniCharPtr inMailServer,
		short inPort /*= EmaiDefaultSmtpPort*/,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpConnect(mObject,
							inMailServer, inPort, inOptions));
}

inline void
SmtpSession::ConnectEx(
		EmaiConstUniCharPtr inMailServer,
		short inPort /*= EmaiDefaultSmtpPort*/,
		const EmaiProxyInfo* inProxyInfo /*= NULL*/,
		void* inUserData,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpConnectEx(mObject,
							inMailServer, inPort, inProxyInfo, inUserData, inOptions));
}

inline void
SmtpSession::Disconnect(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpDisconnect(mObject, inOptions));
}

inline void
SmtpSession::Authenticate(
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inPassword,
		EmaiOptions inOptions /*= EmaiSMTPAuthAutomatic*/
	)
{
	CheckErrorCode(EmaiSmtpAuthenticate(mObject, inAccount,
							inPassword, inOptions));
}

inline void
SmtpSession::SendMessage(
		const Message& inMessage,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpSendMessage(mObject, inMessage, inOptions));
}

inline void
SmtpSession::SendMessageEx(
		const Message& inMessage,
		StringList& outFailedRecipients,
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	EmaiStringList stringList = NULL;
	CheckErrorCode(EmaiSmtpSendMessageEx(mObject, inMessage, &stringList, inOptions));
	if (stringList != NULL)
		outFailedRecipients = StringList(stringList, true);
}

inline void
SmtpSession::Cancel(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpCancel(mObject, inOptions));
}

inline void
SmtpSession::Noop(
		EmaiOptions inOptions /*= EmaiNullOptions*/
	)
{
	CheckErrorCode(EmaiSmtpNoop(mObject, inOptions));
}


}


#endif /*__EMAISMTP_HPP__*/
